/*******************************************************************************
 * de.langzeitarchivierung.kopal.actionmodule / AddDataToDb.java
 * 
 * Copyright 2005-2007 by Project kopal
 *
 * http://kopal.langzeitarchivierung.de/
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307, USA
 * 
 * 
 *******************************************************************************
 *
 *
 * CHANGELOG:
 * 10.07.2007	koLibRI version 1.0
 ******************************************************************************/

package de.langzeitarchivierung.kopal.actionmodule;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import de.langzeitarchivierung.kopal.ProcessData;
import de.langzeitarchivierung.kopal.Status;
import de.langzeitarchivierung.kopal.Step;
import de.langzeitarchivierung.kopal.util.db.MappToHib;

/*******************************************************************************
 * This module adds data of the process to the database. You can store data for
 * each file with or without technical metadata, dublin core metadata and the
 * custom ids of this process.
 * 
 * @author ludwig
 * @version 20070710
 ******************************************************************************/

public class AddDataToDb implements ActionModule
{

	// STATE (Instance variables) **********************************************

	Step			step;
	ProcessData		processData;

	boolean			useDatabase			= false;
	boolean			storeFileData		= false;
	boolean			storeFileDataTechMd	= false;
	boolean			storeDc				= false;
	boolean			storeIds			= false;
	boolean			storeCustomData		= false;
	List<String>	customDataEelements	= new ArrayList<String>();

	// MANIPULATION (Manipulation - what the object does) ******************

	/***************************************************************************
	 * Starts the essential task of this class
	 * 
	 * @see de.langzeitarchivierung.kopal.actionmodule.ActionModule#go()
	 **************************************************************************/
	public void go() {
		if (this.useDatabase) {
			String message = "";

			// Shall the IDs be stored?
			if (this.storeIds) {
				try {
					MappToHib.addIdentifier(this.processData);
					message += " | custom IDs -> identifier";
				}
				catch (Exception e) {
					e.printStackTrace();
					this.step.setStatus(Status.ERROR, "Adding custom Ids "
							+ "to database failed. " + "System message: "
							+ e.getMessage());
					try {
						MappToHib.removeDias(this.processData);
					}
					catch (Exception e1) {
						e1.printStackTrace();
						this.step.setStatus(Status.ERROR, "Removing the DIAS "
								+ "table entry failed. System message: "
								+ e1.getMessage());
						e1.printStackTrace();
					}
					return;
				}
			}

			// Shall the DC data be stored?
			if (this.storeDc) {
				try {
					MappToHib.addDc(this.processData);
					message += " | DC metadata -> dublincore";
				}
				catch (Exception e) {
					e.printStackTrace();
					this.step.setStatus(Status.ERROR, "Adding DC data "
							+ "to database failed. System message: "
							+ e.getMessage());
					try {
						MappToHib.removeDias(this.processData);
					}
					catch (Exception e1) {
						e1.printStackTrace();
						this.step.setStatus(Status.ERROR, "Removing the DIAS "
								+ "table entry failed. System message: "
								+ e1.getMessage());
						e1.printStackTrace();
					}
					return;
				}
			}

			// Shall the fileData be stored?
			if (this.storeFileData) {
				try {
					message += " | file data ";
					MappToHib.addFiles(this.processData,
							this.storeFileDataTechMd);
					if (this.storeFileDataTechMd) {
						message += "including";
					}
					else {
						message += "without";
					}
					message += " techMd -> file";
				}
				catch (Exception e) {
					e.printStackTrace();
					this.step.setStatus(Status.ERROR, "Adding file data "
							+ "to database failed. System message: "
							+ e.getMessage());
					try {
						MappToHib.removeDias(this.processData);
					}
					catch (Exception e1) {
						e1.printStackTrace();
						this.step.setStatus(Status.ERROR, "Removing the DIAS "
								+ "table entry failed. System message: "
								+ e1.getMessage());
						e1.printStackTrace();
					}
					return;
				}
			}

			// Shall the customData be stored (in the various table) and what
			// elements?
			if (this.storeCustomData && !this.customDataEelements.isEmpty()) {
				try {
					message += " | custom data (";
					MappToHib.addVarious(this.processData,
							this.customDataEelements);
					Iterator iter = this.customDataEelements.iterator();
					while (iter.hasNext()) {
						message += (String) iter.next() + " ";
					}
					message = message.trim() + ") -> various";
				}
				catch (Exception e) {
					e.printStackTrace();
					this.step.setStatus(Status.ERROR, "Adding custom data "
							+ "elements to database failed. System message: "
							+ e.getMessage());
					try {
						MappToHib.removeDias(this.processData);
					}
					catch (Exception e1) {
						e1.printStackTrace();
						this.step.setStatus(Status.ERROR, "Removing the DIAS "
								+ "table entry failed. System message: "
								+ e1.getMessage());
						e1.printStackTrace();
					}
					return;
				}
			}

			this.step.setStatus(Status.RUNNING, "Added to database: "
					+ message.substring(2).trim());

			// If added some data correctly, set status to DONE.
			this.step.setStatus(Status.DONE, "Data addition complete");
		}
		else {
			this.step.setStatus(Status.DONE, "Configuration says "
					+ "useDatabase=false, no data added");
		}
	}

	// GET & SET METHODS **************************************************

	/***************************************************************************
	 * @param _processData The processData to set.
	 **************************************************************************/
	public void setProcessData(ProcessData _processData) {
		this.processData = _processData;
	}

	/***************************************************************************
	 * @param _step The step to set.
	 **************************************************************************/
	public void setStep(Step _step) {
		this.step = _step;
	}

	/***************************************************************************
	 * @return boolean - Whether or not the Database shall be used
	 **************************************************************************/
	public boolean isUseDatabase() {
		return this.useDatabase;
	}

	/***************************************************************************
	 * @param _useDatabase - Whether or not the Database shall be used
	 **************************************************************************/
	public void setUseDatabase(boolean _useDatabase) {
		this.useDatabase = _useDatabase;
	}

	/***************************************************************************
	 * @return isStoreDC - Whether or not the Dublin Core Data shall be stored
	 **************************************************************************/
	public boolean isStoreDc() {
		return this.storeDc;
	}

	/***************************************************************************
	 * @param _storeDc - Whether or not the Dublin Core Data shall be stored
	 **************************************************************************/
	public void setStoreDc(boolean _storeDc) {
		this.storeDc = _storeDc;
	}

	/***************************************************************************
	 * @return isStoreFileData - Whether or not the File Data shall be stored
	 **************************************************************************/
	public boolean isStoreFileData() {
		return this.storeFileData;
	}

	/***************************************************************************
	 * @param _storeFileData - Whether or not the File Data shall be stored
	 **************************************************************************/
	public void setStoreFileData(boolean _storeFileData) {
		this.storeFileData = _storeFileData;
	}

	/***************************************************************************
	 * @return isStoreFileDataTechMd - Whether or not the technical metadata
	 *         shall be stored
	 **************************************************************************/
	public boolean isStoreFileDataTechMd() {
		return this.storeFileDataTechMd;
	}

	/***************************************************************************
	 * @param _storeFileDataTechMd - Whether or not the technical metadata shall
	 *            be stored
	 **************************************************************************/
	public void setStoreFileDataTechMd(boolean _storeFileDataTechMd) {
		this.storeFileDataTechMd = _storeFileDataTechMd;
	}

	/***************************************************************************
	 * @return isStoreIds - Whether or not IDs shall be stored
	 **************************************************************************/
	public boolean isStoreIds() {
		return this.storeIds;
	}

	/***************************************************************************
	 * @param _storeIds - Whether or not IDs shall be stored
	 **************************************************************************/
	public void setStoreIds(boolean _storeIds) {
		this.storeIds = _storeIds;
	}

	/***************************************************************************
	 * @return Returns the storeCustomData.
	 **************************************************************************/
	public boolean isStoreCustomData() {
		return this.storeCustomData;
	}

	/***************************************************************************
	 * @param storeCustomData The storeCustomData to set.
	 **************************************************************************/
	public void setStoreCustomData(boolean storeCustomData) {
		this.storeCustomData = storeCustomData;
	}

	/***************************************************************************
	 * @return Returns the customDataEelements.
	 **************************************************************************/
	public List<String> getCustomDataEelements() {
		return this.customDataEelements;
	}

	/***************************************************************************
	 * @param customDataEelements The customDataEelements to set.
	 **************************************************************************/
	public void setCustomDataEelements(String customDataEelement) {
		this.customDataEelements.add(customDataEelement);
	}

}
