/*******************************************************************************
 * de.langzeitarchivierung.kopal.jhove / AskDIASifIngested.java
 *  
 * Copyright 2005-2007 by Project kopal
 * 
 * http://kopal.langzeitarchivierung.de/
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307, USA
 * 
 * 
 ******************************************************************************
 *
 * 
 * CHANGELOG:
 * 10.07.2007	koLibRI version 1.0
 ******************************************************************************/

package de.langzeitarchivierung.kopal.actionmodule;

import java.io.StringReader;

import de.langzeitarchivierung.kopal.ProcessData;
import de.langzeitarchivierung.kopal.Status;
import de.langzeitarchivierung.kopal.Step;
import de.langzeitarchivierung.kopal.retrieval.DiasAccess;
import de.langzeitarchivierung.kopal.retrieval.DiasAccessMetadataResponse;
import de.langzeitarchivierung.kopal.retrieval.DiasAccessResponse;
import de.langzeitarchivierung.kopal.util.Configurator;

/*******************************************************************************
 * Class to ask DIAS if an Object with a given URN already ingested. It sets the
 * status to error. So the WorkflowTool continues with the next SIP.
 * 
 * @author Kadir Karaca Koçer, German National Library
 * @version 20070423
 * @see DiasAccess
 * @since 20070423
 ******************************************************************************/

public class AskDIASifIngested implements ActionModule
{

	// STATE (Instance variables) *****************************************

	ProcessData	processData;
	Step		step;

	// MANIPULATION (Manipulation - what the object does) ******************

	/***************************************************************************
	 * Method to check if an Asset already in DIAS (without using the ticketing
	 * system).
	 * 
	 * @see de.langzeitarchivierung.kopal.actionmodule.ActionModule#go()
	 **************************************************************************/
	public void go() {
		String extId = null;
		String diasStringResponse = null;
		boolean ingested = false;
		try {
			extId = this.processData.getMetadata().getObjectId();
			DiasAccess access = new DiasAccess();
			Configurator.configure(access);

			// dias returns the latest asset with the specified external id.
			diasStringResponse = access.requestShortMdByExtId(extId, "xml",
					false);
			DiasAccessResponse assetData = DiasAccess
					.parseDiasResponse(new StringReader(diasStringResponse));

			// if the internal id of the latest asset is not "error" or
			// "NOT IN DIAS RESPONSE" we assume it is ingested succesfully
			if (assetData != null) {
				if (!assetData.getResponseType().equalsIgnoreCase("error")) {
					DiasAccessMetadataResponse assetMetadata = assetData
							.getMetadata().get(0);
					if (!assetMetadata.getInternalAssetId().equals(
							DiasAccessResponse.NOT_IN_DIAS_RESPONSE)) {
						ingested = true;
					}
				}
			}
		}
		catch (Exception e) {
			this.step.setStatus(Status.WARNING, "Exception in "
					+ "AskDIASifIngested! System message: " + e.getMessage());
			e.printStackTrace();
		}

		// If there is a SIP in DIAS with that URN cancel the operation with an
		// Error
		if (ingested) {
			this.step.setStatus(Status.ERROR, "SIP with URN: " + extId
					+ " is already in DIAS!");
		}
		else {
			this.step.setStatus(Status.DONE, "SIP with URN: " + extId
					+ " is not in DIAS");
		}
	}

	// INTERNAL (Internal - implementation details, local classes, ...) ****

	/***************************************************************************
	 * @param _step The step to set.
	 **************************************************************************/
	public void setStep(Step _step) {
		this.step = _step;
	}

	/***************************************************************************
	 * @param _processData The processData to set.
	 **************************************************************************/
	public void setProcessData(ProcessData _processData) {
		this.processData = _processData;
	}

}
