/*******************************************************************************
 * de.langzeitarchivierung.kopal.actionmodule / CleanUpFiles.java
 * 
 * Copyright 2005-2007 by Project kopal
 * 
 * http://kopal.langzeitarchivierung.de/
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307, USA
 * 
 * 
 *******************************************************************************
 * 
 * 
 * CHANGELOG:
 * 10.07.2007	koLibRI version 1.0
 ******************************************************************************/

package de.langzeitarchivierung.kopal.actionmodule;

import java.io.File;

import de.langzeitarchivierung.kopal.ProcessData;
import de.langzeitarchivierung.kopal.Status;
import de.langzeitarchivierung.kopal.Step;

/*******************************************************************************
 * Deletes the asset and an unpacked mets.xml (named zipFileName with .xml
 * instead of .zip like produced by UnzipMets) in the temp dir.
 * 
 * @author Ludwig
 * @version 20070710
 ******************************************************************************/

public class CleanUpFiles implements ActionModule
{

	// STATE (Instance variables) *****************************************

	ProcessData		processData;
	Step			step;

	private String	tempDir;

	// CREATION (Constructors, factory methods, static/inst init) **********

	/** Constructor */
	public CleanUpFiles() {
		super();
	}

	// MANIPULATION (Manipulation - what the object does) ******************

	/**
	 * @see de.langzeitarchivierung.kopal.actionmodule.ActionModule#go()
	 */
	public void go() {
		this.step.setStatus(Status.RUNNING, "Deleting SIP and (if existing) "
				+ "mets.xml");
		File tempSip = this.processData.getSipLocation();
		String sipName = tempSip.getName();
		File tempMets = null;

		// Determine the name of the mets file, depends on the file format.
		if (sipName.endsWith(".zip") || sipName.endsWith(".tar")) {
			tempMets = new File(this.tempDir, sipName.substring(0, sipName
					.length() - 4)
					+ ".xml");
		}
		else if (sipName.endsWith(".tar.gz")) {
			tempMets = new File(this.tempDir, sipName.substring(0, sipName
					.length() - 7)
					+ ".xml");
		}
		else {
			this.step.setStatus(Status.ERROR, "Unknown SIP file format. "
					+ "Please delete SIP and its METS file manually");
			return;
		}

		// Delete and check if things were deleted correctly.
		if (tempSip.delete()) {
			this.step.setStatus(Status.RUNNING, "Deletion of SIP complete");
		}
		else {
			this.step.setStatus(Status.ERROR, "Deletion of SIP failed. "
					+ "Check the reason and delete SIP manually");
			return;
		}

		if (tempMets.delete()) {
			this.step.setStatus(Status.RUNNING, "Deletion of METS file "
					+ "complete");
		}
		else if (!tempMets.exists()) {
			this.step.setStatus(Status.RUNNING, "No METS file for "
					+ "deletion available");
		}
		else {
			this.step.setStatus(Status.ERROR, "Deletion of METS file "
					+ "failed. Check the reason and delete file manually");
			return;
		}

		this.step.setStatus(Status.DONE, "All existing temporary "
				+ "files deleted");
	}

	// GET & SET METHODS *******************************************************

	/***************************************************************************
	 * @param _tempDir The tempDir to set.
	 **************************************************************************/
	public void setTempDir(String _tempDir) {
		this.tempDir = _tempDir;
	}

	/***************************************************************************
	 * @param _processData The processData to set.
	 **************************************************************************/
	public void setProcessData(ProcessData _processData) {
		this.processData = _processData;
	}

	/***************************************************************************
	 * @param _step The step to set.
	 **************************************************************************/
	public void setStep(Step _step) {
		this.step = _step;
	}

}
