/*******************************************************************************
 * de.langzeitarchivierung.kopal.administration / DiasAdminSRUResponse.java
 *  
 * Copyright 2005-2007 by Project kopal
 * 
 * http://kopal.langzeitarchivierung.de/
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307, USA
 * 
 * 
 ******************************************************************************
 *
 *
 * CHANGELOG:
 * 10.07.2007	koLibRI version 1.0
 ******************************************************************************/

package de.langzeitarchivierung.kopal.administration;

import java.util.ArrayList;
import java.util.List;

/*******************************************************************************
 * Class representing the data model for responses of the Dias Administration
 * Interfaces SRU server according to the "Detailed Specifications for
 * Administration and Search Interfaces to DIAS-Core" Version 1.0 by IBM
 * Nederland N.V. 2000, 2006
 * 
 * @author Matthias Neubauer, Deutsche Nationalbibliothek
 * @version 20070604
 * @since 20070604
 ******************************************************************************/

public class DiasAdminSRUResponse
{

	// STATE (Instance variables) *****************************************

	private String			version				= "";
	private long			numberOfRecords		= 0;
	private String			resultSetId			= "";
	private long			resultSetIdleTime	= 0;
	private List<SRURecord>	records				= new ArrayList<SRURecord>();

	// QUERIES (Queries - no change to object's state) ********************

	/***************************************************************************
	 * @see java.lang.Object#toString()
	 **************************************************************************/
	@Override
	public String toString() {
		String out = "\n   Version: " + this.version;
		out += "\n   Number of Records: " + this.numberOfRecords;
		out += "\n   ResultSetId: " + this.resultSetId;
		out += "\n   ResultSetIdleTime: " + this.resultSetIdleTime;
		out += "\n   Records:";

		for (int i = 0; i < this.records.size(); i++) {
			out += this.records.get(i).toString();
		}

		out += "\n ----------------------";

		return out;
	}

	// GET & SET METHODS **************************************************

	/***************************************************************************
	 * @return Returns the records
	 **************************************************************************/
	public List<SRURecord> getRecords() {
		return this.records;
	}

	/***************************************************************************
	 * @param record The record data to add.
	 **************************************************************************/
	public void addRecord(SRURecord record) {
		this.records.add(record);
	}

	/***************************************************************************
	 * @return Returns the number of records.
	 **************************************************************************/
	public long getNumberOfRecords() {
		return this.numberOfRecords;
	}

	/***************************************************************************
	 * @param _numberOfRecords The number of records to set.
	 **************************************************************************/
	public void setNumberOfRecords(long _numberOfRecords) {
		this.numberOfRecords = _numberOfRecords;
	}

	/***************************************************************************
	 * @return Returns the result set ID.
	 **************************************************************************/
	public String getResultSetId() {
		return this.resultSetId;
	}

	/***************************************************************************
	 * @param _resultSetId The result set ID to set.
	 **************************************************************************/
	public void setResultSetId(String _resultSetId) {
		this.resultSetId = _resultSetId;
	}

	/***************************************************************************
	 * @return Returns the result set idle time.
	 **************************************************************************/
	public long getResultSetIdleTime() {
		return this.resultSetIdleTime;
	}

	/***************************************************************************
	 * @param _resultSetIdleTime The result set idle time to set.
	 **************************************************************************/
	public void setResultSetIdleTime(long _resultSetIdleTime) {
		this.resultSetIdleTime = _resultSetIdleTime;
	}

	/***************************************************************************
	 * @return Returns the version.
	 **************************************************************************/
	public String getVersion() {
		return this.version;
	}

	/***************************************************************************
	 * @param ver The version to set.
	 **************************************************************************/
	public void setVersion(String ver) {
		this.version = ver;
	}

}
