/*******************************************************************************
 * de.langzeitarchivierung.kopal.administration / SRURecord.java
 *  
 * Copyright 2005-2007 by Project kopal
 * 
 * http://kopal.langzeitarchivierung.de/
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307, USA
 * 
 * 
 ******************************************************************************
 *
 *
 * CHANGELOG:
 * 10.07.2007	koLibRI version 1.0
 ******************************************************************************/

package de.langzeitarchivierung.kopal.administration;

/*******************************************************************************
 * Class representing the data model for a SRU record according to the "Detailed
 * Specifications for Administration and Search Interfaces to DIAS-Core" Version
 * 1.0 by IBM Nederland N.V. 2000, 2006
 * 
 * @author Matthias Neubauer, Deutsche Nationalbibliothek
 * @version 20070604
 * @since 20070604
 ******************************************************************************/

public class SRURecord
{

	// STATE (Instance variables) *****************************************

	private String	externalAssetID				= "";
	private String	internalAssetID				= "";
	private String	assetVersion				= "";
	private String	assetDescription			= "";
	private boolean	AIPIsDeleted				= false;
	private String	AIPDeletionTimestamp		= "";
	private String	assetCreationTimestamp		= "";
	private String	assetIngestTimestamp		= "";
	private String	CMOldObjectIdentifier		= "";
	private String	CMOldObjectVersion			= "";
	private long	totalAffectedContentFiles	= 0;

	// QUERIES (Queries - no change to object's state) ********************

	/***************************************************************************
	 * TODO Method to get the value of a specific ExtraResponsData element by
	 * its name.
	 * 
	 * @param elementName - The extra data elements name
	 * @return String - The value of the given data element
	 **************************************************************************/
	public String getExtraDataElementValue(String elementName) {
		String value = "";

		// TODO: Retrieve value!
		System.out.println(elementName);

		return value;
	}

	/***************************************************************************
	 * @see java.lang.Object#toString()
	 **************************************************************************/
	@Override
	public String toString() {
		String out = "\n ---------------------- \n   External Asset ID: "
				+ this.externalAssetID;
		out += "\n   Internal Asset ID: " + this.internalAssetID;
		out += "\n   Asset Version: " + this.assetVersion;
		if (this.assetDescription != null && !this.assetDescription.equals("")) {
			out += "\n   Asset Description: " + this.assetDescription;
		}
		out += "\n   AIP is deleted: " + this.AIPIsDeleted;
		if (this.AIPDeletionTimestamp != null
				&& !this.AIPDeletionTimestamp.equals("")) {
			out += "\n   AIP Deletion Timestamp: " + this.AIPDeletionTimestamp;
		}
		out += "\n   Asset Creation Timestamp: " + this.assetCreationTimestamp;
		out += "\n   Asset Ingest Timestamp: " + this.assetIngestTimestamp;
		if (this.CMOldObjectIdentifier != null
				&& !this.CMOldObjectIdentifier.equals("")) {
			out += "\n   CustomMiogration.oldObjectIdentifier: "
					+ this.CMOldObjectIdentifier;
		}
		if (this.CMOldObjectVersion != null
				&& !this.CMOldObjectVersion.equals("")) {
			out += "\n   CustomMigration.oldObjectVersion: "
					+ this.CMOldObjectVersion;
		}
		if (this.totalAffectedContentFiles > 0) {
			out += "\n   Total Affected Content Files: "
					+ this.totalAffectedContentFiles;
		}

		return out;
	}

	// GET & SET METHODS **************************************************

	/***************************************************************************
	 * @return Returns the AIP deletion timestamp.
	 **************************************************************************/
	public String getAIPDeletionTimestamp() {
		return this.AIPDeletionTimestamp;
	}

	/***************************************************************************
	 * @param _deletionTimestamp The AIP deletion timestamp to set.
	 **************************************************************************/
	public void setAIPDeletionTimestamp(String _deletionTimestamp) {
		this.AIPDeletionTimestamp = _deletionTimestamp;
	}

	/***************************************************************************
	 * @return Returns wether or not the AIP is deleted.
	 **************************************************************************/
	public boolean getAIPIsDeleted() {
		return this.AIPIsDeleted;
	}

	/***************************************************************************
	 * @param _isDeleted Wether the AIP is deleted or not.
	 **************************************************************************/
	public void setAIPIsDeleted(boolean _isDeleted) {
		this.AIPIsDeleted = _isDeleted;
	}

	/***************************************************************************
	 * @return Returns the asset creation timestamp.
	 **************************************************************************/
	public String getAssetCreationTimestamp() {
		return this.assetCreationTimestamp;
	}

	/***************************************************************************
	 * @param _assetCreationTimestamp The asset creation timestamp to set.
	 **************************************************************************/
	public void setAssetCreationTimestamp(String _assetCreationTimestamp) {
		this.assetCreationTimestamp = _assetCreationTimestamp;
	}

	/***************************************************************************
	 * @return Returns the asset Description.
	 **************************************************************************/
	public String getAssetDescription() {
		return this.assetDescription;
	}

	/***************************************************************************
	 * @param _assetDescription The asset description to set.
	 **************************************************************************/
	public void setAssetDescription(String _assetDescription) {
		this.assetDescription = _assetDescription;
	}

	/***************************************************************************
	 * @return Returns the asset ingest timestamp.
	 **************************************************************************/
	public String getAssetIngestTimestamp() {
		return this.assetIngestTimestamp;
	}

	/***************************************************************************
	 * @param _assetIngestTimestamp The asset ingest timestamp to set.
	 **************************************************************************/
	public void setAssetIngestTimestamp(String _assetIngestTimestamp) {
		this.assetIngestTimestamp = _assetIngestTimestamp;
	}

	/***************************************************************************
	 * @return Returns the asset version.
	 **************************************************************************/
	public String getAssetVersion() {
		return this.assetVersion;
	}

	/***************************************************************************
	 * @param _assetVersion The asset version to set.
	 **************************************************************************/
	public void setAssetVersion(String _assetVersion) {
		this.assetVersion = _assetVersion;
	}

	/***************************************************************************
	 * @return Returns the CustomMigration.oldObjectIdentifier.
	 **************************************************************************/
	public String getCMOldObjectIdentifier() {
		return this.CMOldObjectIdentifier;
	}

	/***************************************************************************
	 * @param _oldObjectIdentifier The CustomMigration.oldObjectIdentifier to
	 *            set.
	 **************************************************************************/
	public void setCMOldObjectIdentifier(String _oldObjectIdentifier) {
		this.CMOldObjectIdentifier = _oldObjectIdentifier;
	}

	/***************************************************************************
	 * @return Returns the CustomMigration.OldObjectVersion.
	 **************************************************************************/
	public String getCMOldObjectVersion() {
		return this.CMOldObjectVersion;
	}

	/***************************************************************************
	 * @param _oldObjectVersion The CustomMigration.OldObjectVersion to set.
	 **************************************************************************/
	public void setCMOldObjectVersion(String _oldObjectVersion) {
		this.CMOldObjectVersion = _oldObjectVersion;
	}

	/***************************************************************************
	 * @return Returns the external asset ID.
	 **************************************************************************/
	public String getExternalAssetID() {
		return this.externalAssetID;
	}

	/***************************************************************************
	 * @param _externalAssetID The external asset ID to set.
	 **************************************************************************/
	public void setExternalAssetID(String _externalAssetID) {
		this.externalAssetID = _externalAssetID;
	}

	/***************************************************************************
	 * @return Returns the internal asset ID.
	 **************************************************************************/
	public String getInternalAssetID() {
		return this.internalAssetID;
	}

	/***************************************************************************
	 * @param _internalAssetID The internal asset ID to set.
	 **************************************************************************/
	public void setInternalAssetID(String _internalAssetID) {
		this.internalAssetID = _internalAssetID;
	}

	/***************************************************************************
	 * @return Returns the number of total affected content files.
	 **************************************************************************/
	public long getTotalAffectedContentFiles() {
		return this.totalAffectedContentFiles;
	}

	/***************************************************************************
	 * @param _totalAffectedContentFiles The number of total affected content
	 *            files to set.
	 **************************************************************************/
	public void setTotalAffectedContentFiles(long _totalAffectedContentFiles) {
		this.totalAffectedContentFiles = _totalAffectedContentFiles;
	}

}
