/**********************************************************************
 * de.langzeitarchivierung.kopal.retrieval / DiasIngestResponse.java
 *  
 * Copyright 2005-2007 by Project kopal
 * 
 * http://kopal.langzeitarchivierung.de/
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307, USA
 * 
 * 
 *******************************************************************************
 * 
 * 
 * CHANGELOG:
 * 10.07.2007	koLibRI version 1.0
 ******************************************************************************/

package de.langzeitarchivierung.kopal.ingest;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/*******************************************************************************
 * The DIAS response to ingest requests as a bean. To understand the data fields
 * read the SIP Interface Specification.
 * 
 * @author Funk
 * @version 20070308
 ******************************************************************************/

public class DiasIngestResponse implements Serializable
{

	// CONSTANTS **********************************************************

	private static final long					serialVersionUID		= 1L;

	public static final String					NOT_IN_DIAS_RESPONSE	= "NOT IN DIAS RESPONSE";
	public static final String					RESPONSE_TYPE_BUSY		= "busy";
	public static final String					RESPONSE_TYPE_ERROR		= "error";
	public static final String					RESPONSE_VERSION		= "1.0";

	// STATE (Instance variables) *****************************************

	private String								responseType;
	private String								responseVersion;
	// Between the DESCRIPTION tag.
	private String								description;
	// Between the METADATA tag.
	private List<DiasIngestMetadataResponse>	metadata;

	// CREATION (Constructors, factory methods, static/inst init) **********

	/***************************************************************************
	 * Constructor.
	 **************************************************************************/
	public DiasIngestResponse() {
		super();
		this.metadata = new ArrayList<DiasIngestMetadataResponse>();
	}

	// GET & SET METHODS **************************************************

	/***************************************************************************
	 * Gets the description.
	 * 
	 * @return Returns the description.
	 **************************************************************************/
	public String getDescription() {
		return this.description;
	}

	/***************************************************************************
	 * Sets the description.
	 * 
	 * @param _description The description to set.
	 **************************************************************************/
	public void setDescription(String _description) {
		this.description = _description;
	}

	/***************************************************************************
	 * Gets the list of DiasIngestMetadata objects.
	 * 
	 * @return Returns the metadata.
	 **************************************************************************/
	public List<DiasIngestMetadataResponse> getMetadata() {
		return this.metadata;
	}

	/***************************************************************************
	 * Adds a DiasIngestMetadata object to the metadata list.
	 * 
	 * @param md the metadata object to add.
	 **************************************************************************/
	public void addMetadata(DiasIngestMetadataResponse md) {
		this.metadata.add(md);
	}

	/***************************************************************************
	 * Gets the responseVersion.
	 * 
	 * @return Returns the responseVersion.
	 **************************************************************************/
	public String getResponseVersion() {
		return this.responseVersion;
	}

	/***************************************************************************
	 * Sets the responseVersion.
	 * 
	 * @param _responseVersion The responseVersion to set.
	 **************************************************************************/
	public void setResponseVersion(String _responseVersion) {
		this.responseVersion = _responseVersion;
	}

	/***************************************************************************
	 * Gets the responseType.
	 * 
	 * @return Returns the responseType.
	 **************************************************************************/
	public String getResponseType() {
		return this.responseType;
	}

	/***************************************************************************
	 * Sets the responseType.
	 * 
	 * @param _responseType The responseType to set.
	 **************************************************************************/
	public void setResponseType(String _responseType) {
		this.responseType = _responseType;
	}

	// QUERIES (Queries - no change to object's state) ********************

	/***************************************************************************
	 * @see java.lang.Object#toString()
	 **************************************************************************/
	@Override
	public String toString() {
		String result = "";

		result += "responseType: " + this.responseType;
		result += "\nresponseVersion: " + this.responseVersion;
		result += "\ndescription: " + this.description;

		int j = 0;
		for (Iterator i = this.metadata.iterator(); i.hasNext(); j++) {
			DiasIngestMetadataResponse element = (DiasIngestMetadataResponse) i
					.next();
			result += "\nmetadatablock #" + j + ":\n" + element;
		}

		return result;
	}

}
