/*******************************************************************************
 * de.langzeitarchivierung.kopal.processstarter.sub / MonitorHttpLocationxample.java
 *  
 * Copyright 2005-2007 by Project kopal
 * 
 * http://kopal.langzeitarchivierung.de/
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307, USA
 * 
 * 
 *******************************************************************************
 *
 *
 * CHANGELOG:
 * 10.07.2007	koLibRI version 1.0
 ******************************************************************************/

package de.langzeitarchivierung.kopal.processstarter;

import java.util.HashMap;

import org.apache.commons.httpclient.HttpClient;

/*******************************************************************************
 * The example HTTP monitoring process starter. In this class only the abstract
 * methods are defined. Please edit as you wish.
 * 
 * @author Stefan Funk, SUB Göttingen
 * @version 20070329
 * @see de.langzeitarchivierung.kopal.processstarter
 * @since 20070123
 ******************************************************************************/

public class MonitorHttpLocationExample extends MonitorHttpLocationBase
{

	/*
	 * Think of any name that identifies the object properly. We take the URL of
	 * the source folder here.
	 * 
	 * (non-Javadoc)
	 * 
	 * @see de.langzeitarchivierung.kopal.processstarter.MonitorHttpLocationBase#getProcessName(java.lang.String)
	 */
	@Override
	protected String getProcessName(String currentUrl) {
		return getSourceFolderName(currentUrl);
	}

	/*
	 * Set as much custom identifiers as you want to, all of them are written to
	 * the database.
	 * 
	 * (non-Javadoc)
	 * 
	 * @see de.langzeitarchivierung.kopal.processstarter.MonitorHttpLocationBase#getCustomIdList(java.lang.String)
	 */
	@Override
	protected HashMap<String, String> getCustomIdList(String currentUrl) {
		HashMap<String, String> result = new HashMap<String, String>();
		return result;
	}

	/*
	 * You can specify a persistent identifier here or use a custom action
	 * module to do so later on. We also use the URL of the source folder here.
	 * 
	 * (non-Javadoc)
	 * 
	 * @see de.langzeitarchivierung.kopal.processstarter.MonitorHttpLocationBase#getPersistentIdentifier(java.lang.String)
	 */
	@Override
	public String getPersistentIdentifier(String currentUrl) {
		return getSourceFolderName(currentUrl);
	}

	/*
	 * Please specify here further restrictions of the URLs you wish to add to
	 * the process list.
	 * 
	 * (non-Javadoc)
	 * 
	 * @see de.langzeitarchivierung.kopal.processstarter.MonitorHttpLocationBase#fileIsRestricted(java.io.File)
	 */
	@Override
	protected boolean sipIsRestricted(HttpClient client, String currentUrl) {
		// Restrict file if currentUrl is a file and readDirectoriesOnly is
		// set to TRUE.
		if (!currentUrl.endsWith("/") && this.readDirectoriesOnly) {
			return true;
		}

		return false;
	}

	// INTERNAL (Internal - implementation details, local classes, ...)

	/***************************************************************************
	 * Get the source folder name out of the source URL.
	 * 
	 * @param currentUrl The URL to get the folder name from.
	 * @return String The folder name to return.
	 **************************************************************************/
	private String getSourceFolderName(String currentUrl) {
		String result = "";
		int filenameStart;
		int filenameEnd;

		// Is the current URL a link to a folder?
		if (currentUrl.endsWith("/")) {
			filenameEnd = currentUrl.lastIndexOf("/");
			filenameStart = currentUrl.lastIndexOf("/", filenameEnd - 1);
			result = currentUrl.substring(filenameStart + 1, filenameEnd);
		}
		// Or a link to a file?
		else {
			filenameStart = currentUrl.lastIndexOf("/");
			result = currentUrl.substring(filenameStart + 1);
		}

		return result;
	}

}
