/*******************************************************************************
 * de.langzeitarchivierung.kopal.processstarter / Server.java
 * 
 * Copyright 2005-2007 by Project kopal
 * 
 * http://kopal.langzeitarchivierung.de/
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307, USA
 * 
 * 
 ******************************************************************************
 * 
 * 
 * CHANGELOG:
 * 10.07.2007	koLibRI version 1.0
 ******************************************************************************/

package de.langzeitarchivierung.kopal.processstarter;

import java.io.DataOutputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.logging.Logger;
import java.util.logging.Level;

import de.langzeitarchivierung.kopal.ProcessQueue;
import de.langzeitarchivierung.kopal.util.Configurator;
import de.langzeitarchivierung.kopal.util.KopalClassLoader;

/*******************************************************************************
 * This ProcessStarter starts threads implementing the interface ServerThread
 * for communication with other programms or koLibRI instances. If some module
 * connects to the defaultConnectionPort it receives the input- and output-port
 * of the new thread.
 * 
 * PRECONDITION: An interaction port number must be specified in the main config
 * file and this port must be free to use.
 * 
 * RESULT: The two port numbers to use are sent back to the requesting client
 * loader.
 * 
 * @author Stefan Funk, Jens Ludwig SUB Goettingen
 * @version 20060228
 * @see de.langzeitarchivierung.kopal.processstarter
 * @since 20050628
 ******************************************************************************/

public class Server implements ProcessStarter
{

	// STATE (Instance variables) ******************************************

	private int				defaultConnectionPort	= 0;
	private Logger			defaultLogger			= Logger
															.getLogger("de.langzeitarchivierung.kopal");
	private ProcessQueue	processQueue;
	private Class			serverClass;

	// CREATION (Constructors, factory methods, static/inst init) **********

	/***************************************************************************
	 * Default constructor
	 **************************************************************************/
	public Server() {
		super();
	}

	// MANIPULATION (Manipulation - what the object does) ******************

	/***************************************************************************
	 * @see de.langzeitarchivierung.kopal.processstarter.ProcessStarter#run()
	 **************************************************************************/
	public void run() {
		try {
			// Open socket on the connection port (main config file).
			ServerSocket defaultConnection = new ServerSocket(
					this.defaultConnectionPort);

			this.defaultLogger.log(Level.INFO, "Server started");

			while (true) {
				this.defaultLogger.log(Level.INFO, "Server for class "
						+ this.getServerClassName() + " listens on port "
						+ this.defaultConnectionPort);

				// Start the server.
				Socket clientConnection = defaultConnection.accept();
				DataOutputStream configureClientStream = new DataOutputStream(
						clientConnection.getOutputStream());

				this.defaultLogger.log(Level.INFO, "Incoming request on port "
						+ this.defaultConnectionPort + ", starting a "
						+ this.getServerClassName() + " Thread");

				// Creates an instance of a new ServerThread.
				ServerThread serverThread = (ServerThread) KopalClassLoader
						.getInstanceOfClass(this.serverClass);
				Configurator.configure(serverThread, this);

				// Start the ServerThread as a daemon.
				Thread t = new Thread(serverThread);
				t.setDaemon(true);
				t.start();

				// Send both port numbers back to the requesting asset builder.
				configureClientStream.writeInt(serverThread.getInputPort());
				configureClientStream.writeInt(serverThread.getOutputPort());

				configureClientStream.close();
			}

		}
		catch (Exception e) {
			e.printStackTrace();
			this.defaultLogger.log(Level.SEVERE, "Error running server: " + e);
		}
	}

	// GET & SET METHODS **************************************************

	/***************************************************************************
	 * @param _defaultConnectionPort The interactionPort to set.
	 **************************************************************************/
	public void setDefaultConnectionPort(int _defaultConnectionPort) {
		this.defaultConnectionPort = _defaultConnectionPort;
	}

	/***************************************************************************
	 * @return String
	 **************************************************************************/
	public String getServerClassName() {
		return this.serverClass.getName();
	}

	/***************************************************************************
	 * @param serverClassName
	 **************************************************************************/
	public void setServerClassName(String serverClassName) {
		try {
			this.serverClass = KopalClassLoader
					.getProcessStarter(serverClassName);
		}
		catch (Exception e) {
			this.defaultLogger.log(Level.SEVERE, "Could not set server class: "
					+ e);
			e.printStackTrace();
		}
	}

	/***************************************************************************
	 * @return ProcessQueue
	 **************************************************************************/
	public ProcessQueue getProcessQueue() {
		return this.processQueue;
	}

	/***************************************************************************
	 * @see de.langzeitarchivierung.kopal.processstarter.ProcessStarter#setInput(String)
	 **************************************************************************/
	public void setInput(String input) {
	// Not needed in this class
	}

	/***************************************************************************
	 * @see de.langzeitarchivierung.kopal.processstarter.ProcessStarter#setProcessQueue(de.langzeitarchivierung.kopal.ProcessQueue)
	 **************************************************************************/
	public void setProcessQueue(ProcessQueue _processQueue) {
		this.processQueue = _processQueue;
	}

}
