/*******************************************************************************
 * de.langzeitarchivierung.kopal.processstarter.migrationmanager / CommandLineMigrationEventListener.java
 * 
 * Copyright 2005-2007 by Project kopal
 * 
 * http://kopal.langzeitarchivierung.de/
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307, USA
 *
 *
 *******************************************************************************
 *
 *
 * CHANGELOG:
 * 10.07.2007	koLibRI version 1.0
 ******************************************************************************/

package de.langzeitarchivierung.kopal.processstarter.migrationmanager;

import java.util.List;

/*******************************************************************************
 * MigrationEventListener which interprets the command line arguments as source
 * and destination object character.
 * 
 * @author Ludwig
 * @version 0.5
 ******************************************************************************/
public class CommandLineMigrationEventListener implements
		MigrationEventListener
{

	// STATE (Instance variables) ******************************************

	MigrationEventScheduler	migEvScheduler	= null;
	List<String>			commandLine		= null;

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Runnable#run()
	 */
	public void run() {
		ObjCharacter srcObjCharacter = null;
		ObjCharacter dstObjCharacter = null;

		// if only two arguments are provided, we assume that it is the standard
		// file format migration request
		if (this.commandLine.size() == 2) {
			srcObjCharacter = new ObjCharacter(ObjCharacter.FILE_FORMAT_QUERY,
					this.commandLine.get(0));
			dstObjCharacter = new ObjCharacter(ObjCharacter.FILE_FORMAT_QUERY,
					this.commandLine.get(1));
		}
		else if (this.commandLine.size() == 4) {
			srcObjCharacter = new ObjCharacter(this.commandLine.get(0),
					this.commandLine.get(1));
			dstObjCharacter = new ObjCharacter(this.commandLine.get(2),
					this.commandLine.get(3));
		}
		else {
			// TODO it is not nice to throw an RuntimeException. Other
			// possibilities?
			throw new RuntimeException("Illegal number of arguments");
		}

		this.migEvScheduler.startMigrationEventConsequences(srcObjCharacter,
				dstObjCharacter);
	}

	/***************************************************************************
	 * (non-Javadoc)
	 * 
	 * @see de.langzeitarchivierung.kopal.processstarter.migrationmanager.MigrationEventListener#setMigrationEventScheduler(de.langzeitarchivierung.kopal.processstarter.migrationmanager.MigrationEventScheduler)
	 **************************************************************************/
	public void setMigrationEventScheduler(
			MigrationEventScheduler migEvSchedulerParam) {
		this.migEvScheduler = migEvSchedulerParam;
	}

	/***************************************************************************
	 * Sets the command line arguments.
	 * 
	 * @param commandLine list of command line arguments as strings
	 **************************************************************************/
	public void setCommandLine(List<String> commandLineParam) {
		this.commandLine = commandLineParam;
	}

}
