/*******************************************************************************
 * de.langzeitarchivierung.kopal.processstarter.migrationmanager / DbObjCharacterTranslator.java
 * 
 * Copyright 2005-2007 by Project kopal
 * 
 * http://kopal.langzeitarchivierung.de/
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307, USA
 *
 *
 *******************************************************************************
 *
 *
 * CHANGELOG:
 * 10.07.2007	koLibRI version 1.0
 ******************************************************************************/

package de.langzeitarchivierung.kopal.processstarter.migrationmanager;

import java.io.File;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.apache.xmlbeans.XmlException;

import de.langzeitarchivierung.kopal.formats.Uof;
import de.langzeitarchivierung.kopal.util.Configurator;
import de.langzeitarchivierung.kopal.util.db.*;

/*******************************************************************************
 * Queries the koLibRI database for the characterised objects.
 * 
 * @author Ludwig
 * @version 0.5
 ******************************************************************************/
public class DbObjCharacterTranslator implements ObjCharacterTranslator
{

	// STATE (Instance variables) ******************************************

	protected static Logger				defaultLogger	= Logger
																.getLogger("de.langzeitarchivierung.kopal");

	private String						modulePackages	= "de.langzeitarchivierung.kopal.util.db";
	private String						mappingFile		= "config/MapUofToDB.xml";
	private FileObjCharacterTranslator	mapper			= null;

	/***************************************************************************
	 * Constructor which uses the default mapping file.
	 * 
	 * @throws Exception If self configuration or parsing of default mappingFile
	 *             fails
	 **************************************************************************/
	public DbObjCharacterTranslator() throws Exception {
		super();

		Configurator.configure(this);
		this.mapper = new FileObjCharacterTranslator(this.mappingFile);
	}

	/***************************************************************************
	 * Constructor which uses the specified mapping file.
	 * 
	 * @param mappingFileParam Path to the UOF-database mapping file
	 * @throws Exception If self configuration or parsing of mappingFile fails
	 **************************************************************************/
	public DbObjCharacterTranslator(String mappingFileParam) throws Exception {
		super();

		Configurator.configure(this);
		this.mapper = new FileObjCharacterTranslator();
		this.mapper.addMetsFile(new File(mappingFileParam));
	}



	/*
	 * (non-Javadoc)
	 * 
	 * @see de.langzeitarchivierung.kopal.processstarter.migrationmanager.ObjCharacterTranslator#getCharacterisedObjectValue(java.lang.String,
	 *      de.langzeitarchivierung.kopal.processstarter.migrationmanager.ObjCharacter)
	 */
	public List<String> getCharacterisedObjectValue(String internalId,
			ObjCharacter objCharacter) throws UnsupportedOperationException {
		List<String> results = new ArrayList<String>();
		System.out.println("getCharacterisedObjectValue: " + objCharacter);
		try {

			List<Dias> dbHits = DBAccess.getObjects(Dias.class, "diasintid",
					internalId);
			if (dbHits.isEmpty()) {
				return null;
			}

			for (Iterator iter = dbHits.iterator(); iter.hasNext();) {
				Dias diasIntIdHit = (Dias) iter.next();

				List<Object> dbObjectsToQuery = new ArrayList<Object>();

				String[] dbPositions = this.getMapping(objCharacter);
				String dbTable = dbPositions[0];
				String dbColumn = dbPositions[1];
				String dbSubColumn = null;
				// if not only a database table and column have been specified
				if (dbPositions.length > 2) {
					dbSubColumn = dbPositions[2];
				}

				// we now check which database table is specified by the mapping
				if (dbTable.equalsIgnoreCase("input")) {
					dbObjectsToQuery.add(diasIntIdHit.getInput());

				}
				else if (dbTable.equalsIgnoreCase("format")) {
					Set<de.langzeitarchivierung.kopal.util.db.File> files = diasIntIdHit
							.getInput().getFiles();
					// check all files
					for (Iterator iterator = files.iterator(); iterator
							.hasNext();) {
						de.langzeitarchivierung.kopal.util.db.File file = (de.langzeitarchivierung.kopal.util.db.File) iterator
								.next();
						Set<Fileformat> fileFormats = file.getFileformats();
						Set<Integer> fileFormatIds = new HashSet<Integer>();
						// get all file formatIds
						for (Iterator ffIterator = fileFormats.iterator(); ffIterator
								.hasNext();) {
							Fileformat fformat = (Fileformat) ffIterator.next();
							fileFormatIds.add(fformat.getFileformatid());
						}
						// get all format objects and add them
						for (Iterator ffIdIterator = fileFormatIds.iterator(); ffIdIterator
								.hasNext();) {
							int formatId = (Integer) ffIdIterator.next();
							dbObjectsToQuery.add((Format) DBAccess
									.getObjectPerId(Format.class, formatId));
						}
					}

				}
				else if (dbTable.equalsIgnoreCase("dublincore")) {
					// get all dublincore metadata elements for this object
					Set<Dublincore> dcs = diasIntIdHit.getInput()
							.getDublincores();
					// remove all unwanted dublincore types
					for (Iterator dcIter = dcs.iterator(); dcIter.hasNext();) {
						Dublincore dc = (Dublincore) dcIter.next();
						// if it isn't the requested dublincore type
						if (dc.getDcelement().getDcelementid() != Integer
								.getInteger(dbSubColumn).intValue()) {
							dcIter.remove();
						}
					}
					dbObjectsToQuery.addAll(dcs);

				}
				else if (dbTable.equalsIgnoreCase("dias")) {
					dbObjectsToQuery.add(diasIntIdHit);

				}
				else if (dbTable.equalsIgnoreCase("file")) {
					dbObjectsToQuery.addAll(diasIntIdHit.getInput().getFiles());

				}
				else if (dbTable.equalsIgnoreCase("identifier")) {
					dbObjectsToQuery.add(diasIntIdHit.getInput()
							.getIdentifiers());

				}
				else if (dbTable.equalsIgnoreCase("various")) {
					dbObjectsToQuery
							.add(diasIntIdHit.getInput().getVariouses());

					// TODO make it possible to search only for a substring in
					// the techMd
				}
				else if (dbTable.equalsIgnoreCase("techmd")) {
					Set<de.langzeitarchivierung.kopal.util.db.File> files = diasIntIdHit
							.getInput().getFiles();
					// check all files
					for (Iterator iterator = files.iterator(); iterator
							.hasNext();) {
						de.langzeitarchivierung.kopal.util.db.File file = (de.langzeitarchivierung.kopal.util.db.File) iterator
								.next();
						dbObjectsToQuery.addAll(file.getTechmds());
					}

					// we have no mapping for this
				}
				else {
					throw new UnsupportedOperationException(
							"Database mapping doesn't allow " + "checking for "
									+ dbTable + "/" + dbColumn);
				}

				// add the resulting values of all objects
				for (Iterator dbObjsToQueryIterator = dbObjectsToQuery
						.iterator(); dbObjsToQueryIterator.hasNext();) {
					Object dbObjToQuery = (Object) dbObjsToQueryIterator.next();
					// TODO dbColumn has to be something that an getMethod
					// exists for
					results.add(getColumnValue(dbObjToQuery, dbColumn)
							.toString());

				}


			}

		}
		catch (Exception e) {
			defaultLogger.log(Level.SEVERE, "Error querying database: "
					+ e.getMessage());
			// TODO: handle exception
			e.printStackTrace();
			return null;
		}

		System.out.println("getCharObjValue #" + results.size());
		return results;
	}

	/*
	 * (non-Javadoc) // TODO how can we characterise embedded formats?
	 * 
	 * @see de.langzeitarchivierung.kopal.processstarter.migrationmanager.ObjCharacterTranslator#containsFormatEmbedded(java.lang.String,
	 *      java.lang.Object)
	 */
	public boolean containsFormatEmbedded(String externalFormatId,
			Object properties) throws UnsupportedOperationException {
		throw new UnsupportedOperationException("Not implemented");
		// return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.langzeitarchivierung.kopal.processstarter.migrationmanager.ObjCharacterTranslator#getCharacterisedObjects(de.langzeitarchivierung.kopal.processstarter.migrationmanager.ObjCharacter)
	 */
	public Set<String> getCharacterisedObjects(ObjCharacter objCharacter)
			throws UnsupportedOperationException {
		Set<String> results = new HashSet<String>();

		try {
			String[] dbPosition = this.getMapping(objCharacter);
			String dbTable = dbPosition[0];
			String dbColumn = dbPosition[1];
			String dbSubColumn = null;
			// if not only a database table and column have been specified
			if (dbPosition.length > 2) {
				dbSubColumn = dbPosition[2];
			}
			String requestedValue = objCharacter.getCharacterisationValue();

			// special case checksum: check dias.checksum_new, then
			// input.checksum
			if (dbColumn.equalsIgnoreCase("checksum_new")
					|| dbColumn.equalsIgnoreCase("checksum")) {
				List<Dias> dbDias = (List<Dias>) DBAccess.getObjects(
						Dias.class, "checksum_new", requestedValue);
				for (Iterator iterator = dbDias.iterator(); iterator.hasNext();) {
					String diasIntId = ((Dias) iterator.next()).getDiasintid();
					if (diasIntId == null) {
						// TODO we can't throw an exception here?
						throw new Exception("Ingest in process for inputid "
								+ diasIntId);
					}
					else {
						results.add(diasIntId);
					}
				}
				List<Input> dbInput = (List<Input>) DBAccess.getObjects(
						Input.class, "checksum", requestedValue);
				results.addAll(getDiasIntIdsForInput(dbInput));

			}
			else if (dbTable.equalsIgnoreCase("dias")) {
				List<Dias> dbDias = (List<Dias>) DBAccess.getObjects(
						Dias.class, dbColumn, requestedValue);

				for (Iterator iterator = dbDias.iterator(); iterator.hasNext();) {
					Dias diasHit = (Dias) iterator.next();
					results.addAll(getDiasIntIdsForInput(diasHit.getInput()));
				}

			}
			else if (dbTable.equalsIgnoreCase("format")) {
				Format dbFormat = (Format) DBAccess.getUniqueObject(
						Format.class, dbColumn, requestedValue);
				Set requestedFf = dbFormat.getFileformats();
				Set files = new HashSet<de.langzeitarchivierung.kopal.util.db.File>();
				for (Iterator reqFfIter = requestedFf.iterator(); reqFfIter
						.hasNext();) {
					Fileformat reqFf = (Fileformat) reqFfIter.next();
					files.add(reqFf.getFile());
				}
				results = getDiasIntIds(files);

			}
			else if (dbTable.equalsIgnoreCase("dublincore")) {
				List dbObjects = DBAccess.getObjects(Dublincore.class,
						dbColumn, requestedValue, "dcelement.dcelementid",
						new Integer(dbSubColumn));
				results = getDiasIntIds(dbObjects);

			}
			else if (dbTable.equalsIgnoreCase("file")) {
				if (dbColumn.equalsIgnoreCase("filename")
						&& !requestedValue
								.startsWith(Uof.FILE_LOCAT_HREF_PREFIX)) {
					requestedValue = Uof.FILE_LOCAT_HREF_PREFIX
							+ requestedValue;
				}
				List dbObjects = DBAccess.getObjects(File.class, dbColumn,
						requestedValue);
				results = getDiasIntIds(dbObjects);

				// This allows a simple default checking of all other tables
				// which link to
				// the input table
			}
			else {
				try {
					List dbObjects = DBAccess.getObjects(getClass(dbTable),
							dbColumn, requestedValue);
					results = getDiasIntIds(dbObjects);
				}
				catch (Exception e) {
					throw new UnsupportedOperationException(
							"Database mapping doesn't allow " + "checking for "
									+ dbTable + "/" + dbColumn + ". Error: "
									+ e.getMessage());
				}
			}
		}
		catch (Exception e) {
			defaultLogger.log(Level.SEVERE, "Error querying database: "
					+ e.getMessage());
			// TODO besser machen
			e.printStackTrace();
			return null;
		}
		return results;
	}

	/*
	 * (non-Javadoc) // checks if object has no successor
	 * 
	 * @see de.langzeitarchivierung.kopal.processstarter.migrationmanager.ObjCharacterTranslator#isNotMigrated(java.lang.String)
	 */
	public boolean isNotMigrated(String internalId) {
		ObjCharacter migObjCharacter = new ObjCharacter(
				ObjCharacter.MIGRATION_QUERY, internalId);
		if (!getCharacterisedObjects(migObjCharacter).isEmpty()) {
			return false;
		}
		return true;
	}

	/***************************************************************************
	 * Returns the internal ids belonging to database entries.
	 * 
	 * @param dbObjects The database entries
	 * @return Set of internal ids belonging
	 * @throws Exception
	 **************************************************************************/
	private Set<String> getDiasIntIds(Collection<Object> dbObjects)
			throws Exception {
		Set<String> result = new HashSet<String>();
		for (Iterator iter = dbObjects.iterator(); iter.hasNext();) {
			Object dbObj = (Object) iter.next();
			System.out.println("dbObjClassName: " + dbObj.getClass().getName());
			result.addAll(getDiasIntIdsForInput(getInput(dbObj)));
		}
		return result;
	}


	/***************************************************************************
	 * Returns the input database entries belonging to a database entry.
	 * 
	 * @param dbObject database entry
	 * @return Set of input objects
	 * @throws NoSuchMethodException
	 * @throws InvocationTargetException
	 * @throws IllegalAccessException
	 **************************************************************************/
	private Set<Input> getInput(Object dbObject) throws NoSuchMethodException,
			InvocationTargetException, IllegalAccessException {
		System.out.println("CLASSNAME: " + dbObject.getClass().getName());
		Set<Input> result = new HashSet<Input>();
		System.out.println(dbObject.getClass());
		System.out.println(Input.class.getName());
		if (dbObject.getClass().equals(Input.class)) {
			result.add((Input) dbObject);
		}
		else {
			Class dbClass = dbObject.getClass();
			// Get the method.
			Method getInput = null;
			Object args[] = {};
			try {
				getInput = dbClass.getMethod("getInput");
				// Invoke the method with the correct value.
				result.add((Input) getInput.invoke(dbObject, args));
			}
			catch (NoSuchMethodException e) {
				e.printStackTrace();
				// now try getInputs() instead of getInput()
				// (for owner and server requests for example)
				getInput = dbClass.getMethod("getInputs");
				result = (Set<Input>) getInput.invoke(dbObject, args);
			}
		}
		return result;
	}

	/***************************************************************************
	 * Return the value of the specified database entry.
	 * 
	 * @param dbObject the database row
	 * @param columnName the database column name
	 * @return value in the database column and row
	 * @throws NoSuchMethodException
	 * @throws InvocationTargetException
	 * @throws IllegalAccessException
	 **************************************************************************/
	private Object getColumnValue(Object dbObject, String columnName)
			throws NoSuchMethodException, InvocationTargetException,
			IllegalAccessException {
		Class dbClass = dbObject.getClass();
		char firstCharUpperCase = Character.toUpperCase(columnName.charAt(0));
		StringBuffer columnNameUpperCase = new StringBuffer(columnName);
		columnNameUpperCase.setCharAt(0, firstCharUpperCase);

		// Get the method.
		Method getColumn = dbClass.getMethod("get" + columnNameUpperCase);
		Object args[] = {};


		// Invoke the method with the correct value.
		return getColumn.invoke(dbObject, args);
	}



	/***************************************************************************
	 * Returns the internal ids of a database input object.
	 * 
	 * @param dbInput database input object
	 * @return set of internal ids
	 * @throws Exception if database query fails
	 **************************************************************************/
	private Set<String> getDiasIntIdsForInput(Input dbInput) throws Exception {
		HashSet param = new HashSet();
		param.add(dbInput);
		return getDiasIntIds(param);
	}

	/***************************************************************************
	 * Returns the internal ids for a list of database input object.
	 * 
	 * @param dbInput list of database input object
	 * @return set of internal ids
	 * @throws Exception if database query fails
	 **************************************************************************/
	private Set<String> getDiasIntIdsForInput(List<Input> dbInputs)
			throws Exception {
		HashSet param = new HashSet();
		param.addAll(dbInputs);
		return getDiasIntIds(param);
	}

	/***************************************************************************
	 * Returns the internal ids for a set of database input object.
	 * 
	 * @param dbInput set of database input object
	 * @return set of internal ids
	 * @throws Exception if database query fails
	 **************************************************************************/
	private Set<String> getDiasIntIdsForInput(Set<Input> dbInputs)
			throws Exception {
		Set<String> result = new HashSet<String>();
		for (Iterator iter = dbInputs.iterator(); iter.hasNext();) {
			Input dbInput = (Input) iter.next();

			Set<Dias> ingestedObjects = dbInput.getDiases();

			// if object has been ingested more than once
			if (ingestedObjects.size() > 1) {
				throw new Exception("Object with the inputId "
						+ dbInput.getInputid()
						+ " has been ingested more than once!");
				// if object hasn't been ingested
			}
			else if (ingestedObjects.size() == 0) {
				throw new Exception("Object with the inputId "
						+ dbInput.getInputid() + " hasn't been ingested!");
				// object has been ingested or ingest is in process
			}
			else {
				String diasIntId = ((Dias) ingestedObjects.iterator().next())
						.getDiasintid();
				if (diasIntId == null) {
					defaultLogger.log(Level.WARNING, "Ingest in process for "
							+ "inputid " + dbInput.getInputid()
							+ ", ignoring it!");
				}
				else {
					result.add(diasIntId);
				}
			}
		}
		return result;
	}



	/***************************************************************************
	 * Loads and returns the class for a class name.
	 * 
	 * @param className name of the class to load
	 * @return the requested class
	 * @throws Exception if loading of class fails
	 **************************************************************************/
	private Class getClass(String className) throws Exception {
		Class tableClass = ClassLoader.getSystemClassLoader().loadClass(
				this.modulePackages + "." + className);
		return tableClass;
	}

	/***************************************************************************
	 * Returns the dot seperated strings of the database mapping for this object
	 * character
	 * 
	 * @param objCharacter object characterisation with the property/query for
	 *            which a mapping is needed
	 * @return the mapping strings
	 * @throws UnsupportedOperationException if no well defined mapping exists
	 * @throws XmlException if accessing the mapping file fails
	 **************************************************************************/
	private String[] getMapping(ObjCharacter objCharacter) throws Exception {
		List<String> mappingResults = this.mapper.getCharacterisedObjectValue(
				null, objCharacter);

		if (mappingResults.isEmpty()) {
			throw new UnsupportedOperationException("No database mapping for "
					+ objCharacter);
		}
		if (mappingResults.size() > 1) {
			throw new UnsupportedOperationException(
					"More than one mapping for " + objCharacter);
		}

		String dbPositionString = mappingResults.get(0);
		return dbPositionString.split("\\.");

	}

	/***************************************************************************
	 * Sets the database mapping file.
	 * 
	 * @param mappingFile
	 **************************************************************************/
	public void setMappingFile(String mappingFile) throws Exception {
		this.mapper = new FileObjCharacterTranslator(mappingFile);
	}

}
