/*******************************************************************************
 * de.langzeitarchivierung.kopal.processstarter.migrationmanager / DiasCQLObjCharacterTranslator.java
 * 
 * Copyright 2005-2007 by Project kopal
 * 
 * http://kopal.langzeitarchivierung.de/
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307, USA
 *
 *
 *******************************************************************************
 *
 *
 * CHANGELOG:
 * 10.07.2007	koLibRI version 1.0
 ******************************************************************************/

package de.langzeitarchivierung.kopal.processstarter.migrationmanager;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.HashSet;
import java.util.Set;
import java.util.List;

import de.langzeitarchivierung.kopal.administration.DiasAdmin;
import de.langzeitarchivierung.kopal.administration.DiasAdminSRUResponse;
import de.langzeitarchivierung.kopal.administration.SRURecord;
import de.langzeitarchivierung.kopal.util.Configurator;

/*******************************************************************************
 * Class for using the administration interface of DIAS according to the
 * "Detailed Specifications for Administration and Search Interfaces to
 * DIAS-Core" Version 1.0 by IBM Nederland N.V. 2000, 2006
 * 
 * @author Matthias Neubauer, Deutsche Nationalbibliothek
 * @version 0.5
 * @since 20070604
 ******************************************************************************/

public class DiasCQLObjCharacterTranslator implements ObjCharacterTranslator
{

	/***************************************************************************
	 * Returns the value of the requested property of the requested object.
	 * 
	 * @param internalId the internal id of the object to query. If it is null
	 *            or empty all available objects are queried.
	 * @param objCharacter The object property to query. The characterisation
	 *            value is ignored.
	 * @return the values of the requested object property. This may be more
	 *         than one.
	 * @throws UnsupportedOperationException If this property cannot be queried
	 *             in the implemented data managemetn
	 **************************************************************************/
	public List<String> getCharacterisedObjectValue(String internalId,
			ObjCharacter objCharacter) throws UnsupportedOperationException {
		throw new UnsupportedOperationException(
				"This functionality has not yet been implemented!");
	}

	/***************************************************************************
	 * Checks if an object has no successor.
	 * 
	 * @param internalId The internal id of the object to check.
	 * @return true if the object with the specified internal id has no
	 *         successor, otherwise false.
	 **************************************************************************/
	public boolean isNotMigrated(String internalId) {

		DiasAdmin admin = new DiasAdmin();
		try {
			Configurator.configure(admin);
		}
		catch (InvocationTargetException e) {
			e.printStackTrace();
		}
		catch (IllegalAccessException e) {
			e.printStackTrace();
		}

		DiasAdminSRUResponse response = null;

		String cqlquery = "dias.InternalAssetID=\"" + internalId + "\"";

		try {
			response = admin.searchAIPsByCQL(cqlquery);
		}
		catch (IOException e) {
			e.printStackTrace();
		}

		if (response != null) {

			List records = response.getRecords();

			if (records.size() > 0) {
				// There should be only 1 record to an internalAssetID
				SRURecord record = (SRURecord) records.get(0);

				if (!record.getCMOldObjectIdentifier().equals("")
						|| !record.getCMOldObjectVersion().equals("")) {
					return false;
				}
			}
		}

		return true;
	}

	/***************************************************************************
	 * Checks if an object has no successor.
	 * 
	 * @param asset The assets record information.
	 * @return true if the object with the specified internal id has no
	 *         successor, otherwise false.
	 **************************************************************************/
	public boolean isNotMigrated(SRURecord asset) {
		if (!asset.getCMOldObjectIdentifier().equals("")
				|| !asset.getCMOldObjectVersion().equals("")) {
			return false;
		}

		return true;
	}

	/***************************************************************************
	 * Returns all internal ids of the characterized objects
	 * 
	 * @param objCharacter
	 * @return the set of internal ids
	 * @throws UnsupportedOperationException If the object characterization
	 *             requires to query a data field which is not supported by the
	 *             data management or translator class.
	 **************************************************************************/
	public Set<String> getCharacterisedObjects(ObjCharacter objCharacter)
			throws UnsupportedOperationException {

		DiasAdmin admin = new DiasAdmin();
		try {
			Configurator.configure(admin);
		}
		catch (InvocationTargetException e) {
			e.printStackTrace();
		}
		catch (IllegalAccessException e) {
			e.printStackTrace();
		}

		DiasAdminSRUResponse response = null;

		try {
			response = admin.searchAIPsByCQL(objCharacter
					.getCharacterisationQuery());
		}
		catch (IOException e) {
			e.printStackTrace();
		}

		if (response != null) {
			Set<String> records = new HashSet<String>();

			List<SRURecord> recordlist = response.getRecords();

			for (int i = 0; i < response.getRecords().size(); i++) {
				records.add(recordlist.get(i).getInternalAssetID());
			}

			return records;
		}

		return null;
	}

}
