/*******************************************************************************
 * de.langzeitarchivierung.kopal.processstarter.migrationmanager / MigrationServiceRegistry.java
 * 
 * Copyright 2005-2007 by Project kopal
 * 
 * http://kopal.langzeitarchivierung.de/
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307, USA
 *
 *
 *******************************************************************************
 *
 *
 * CHANGELOG:
 * 10.07.2007	koLibRI version 1.0
 ******************************************************************************/

package de.langzeitarchivierung.kopal.processstarter.migrationmanager;

import java.io.File;
import java.io.IOException;

import org.apache.xmlbeans.XmlException;

import koLibRIMigrationServiceRegistry.MigrationServicesDocument;
import koLibRIMigrationServiceRegistry.CriteriaDocument.Criteria;
import koLibRIMigrationServiceRegistry.MigrationServiceDocument.MigrationService;
import koLibRIMigrationServiceRegistry.MigrationServicesDocument.MigrationServices;

/*******************************************************************************
 * Manages services for file migration.
 * 
 * @author ludwig
 * @version 0.5
 ******************************************************************************/

public class MigrationServiceRegistry
{

	// STATE (Instance variables) *****************************************

	MigrationService[]	migServices	= null;

	// MANIPULATION (Manipulation - what the object does) *****************

	/***************************************************************************
	 * Constructor which reads the MigrationServiceRegistry xml file.
	 * 
	 * @param migServFile
	 * @throws XmlException
	 * @throws IOException
	 **************************************************************************/
	public MigrationServiceRegistry(File migServFile) throws XmlException,
			IOException {
		super();
		MigrationServicesDocument migServDoc = MigrationServicesDocument.Factory
				.parse(migServFile);
		MigrationServices migServs = migServDoc.getMigrationServices();
		this.migServices = migServs.getMigrationServiceArray();
	}

	/***************************************************************************
	 * Returns the MigrationService requested by name.
	 * 
	 * @param serviceName
	 * @return
	 **************************************************************************/
	public MigrationService getServiceByName(String serviceName) {
		for (int i = 0; i < this.migServices.length; i++) {
			if (this.migServices[i].getServiceName().equalsIgnoreCase(
					serviceName)) {
				return this.migServices[i];
			}
		}

		return null;
	}

	/***************************************************************************
	 * Returns the first MigrationService which matches the specified service
	 * type.
	 * 
	 * @param serviceType
	 * @return
	 **************************************************************************/
	public MigrationService getServiceByType(String[] serviceTypes) {
		for (int i = 0; i < this.migServices.length; i++) {
			if (checkServiceType(this.migServices[i], serviceTypes)) {
				return this.migServices[i];
			}
		}

		return null;
	}

	/***************************************************************************
	 * Returns the first MigrationService which matches one specified source and
	 * destination object character.
	 * 
	 * TODO should this method check the srcObjChr or check the real object if
	 * it fits?
	 * 
	 * @param srcObjChr
	 * @param dstObjChr
	 * @return
	 **************************************************************************/
	public MigrationService getServiceByCriteria(ObjCharacter srcObjChr,
			ObjCharacter dstObjChr) {
		for (int i = 0; i < this.migServices.length; i++) {

			// we have to check that both criteria exist because they are
			// optional
			if (this.migServices[i].getSourceCriteria() != null
					&& this.migServices[i].getDestinationCriteria() != null) {
				Criteria[] srcMigServProps = this.migServices[i]
						.getSourceCriteria().getCriteriaArray();
				Criteria[] dstMigServProps = this.migServices[i]
						.getDestinationCriteria().getCriteriaArray();

				for (int k = 0; k < srcMigServProps.length; k++) {
					ObjCharacter serviceSrcObjChr = new ObjCharacter(
							srcMigServProps[k]);
					if (serviceSrcObjChr.equals(srcObjChr)) {

						for (int j = 0; j < dstMigServProps.length; j++) {
							ObjCharacter serviceDstObjChr = new ObjCharacter(
									dstMigServProps[j]);
							if (serviceDstObjChr.equals(dstObjChr)) {
								return this.migServices[i];
							}
						}
					}
				}
			}
		}

		return null;
	}

	// INTERNAL (Internal - implementation details, local classes, ...) **

	/***************************************************************************
	 * Checks if the migration service fulfills the specified requirements.
	 * 
	 * @param serviceToCheck
	 * @param requiredServiceTypes
	 * @return
	 **************************************************************************/
	private boolean checkServiceType(MigrationService serviceToCheck,
			String[] requiredServiceTypes) {
		for (int j = 0; j < requiredServiceTypes.length; j++) {
			String serviceType = requiredServiceTypes[j];

			// ignore whitespaces
			if (serviceType.length() != 0) {
				boolean serviceRequirementsFulfilled = false;
				for (int k = 0; k < serviceToCheck.getServiceTypeArray().length; k++) {
					if (serviceToCheck.getServiceTypeArray(k).equalsIgnoreCase(
							serviceType)) {
						serviceRequirementsFulfilled = true;
						break;
					}
				}
				if (!serviceRequirementsFulfilled) {
					return false;
				}
			}
		}

		return true;
	}

}
