/*******************************************************************************
 * de.langzeitarchivierung.kopal.processstarter.migrationmanager / ObjCharacter.java
 * 
 * Copyright 2005-2007 by Project kopal
 * 
 * http://kopal.langzeitarchivierung.de/
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307, USA
 *
 *
 *******************************************************************************
 *
 *
 * CHANGELOG:
 * 10.07.2007	koLibRI version 1.0
 ******************************************************************************/

package de.langzeitarchivierung.kopal.processstarter.migrationmanager;

import koLibRIMigrationPolicyTemplates.CriteriaDocument.Criteria;

/*******************************************************************************
 * Characterises an object. An Object is characterised by an xpath query on the
 * uof mets and an optional result.
 * 
 * @author Ludwig
 * @version 0.5
 ******************************************************************************/

public class ObjCharacter
{

	// CONSTANTS **********************************************************

	// uof namespaces
	public static final String	METS_NAMESPACE				= "declare namespace m='http://www.loc.gov/METS/';";
	public static final String	JHOVE_NAMESPACE				= "declare namespace jv='http://hul.harvard.edu/ois/xml/ns/jhove';";
	public static final String	LMER_OBJECT_NAMESPACE		= "declare namespace lo='http://www.ddb.de/LMERobject';";
	public static final String	LMER_FILE_NAMESPACE			= "declare namespace lf='http://www.ddb.de/LMERfile';";
	public static final String	LMER_PROCESS_NAMESPACE		= "declare namespace lp='http://www.ddb.de/LMERprocess';";

	// standard queries
	// TODO is it better to leave the namespaces out?
	public static final String	FILE_FORMAT_QUERY			= "//*[name()='lmerFile:format']";
	// METS_NAMESPACE + LMER_FILE_NAMESPACE +
	// "/m:mets/m:amdSec/m:techMD/m:mdWrap/m:xmlData/lf:format/text()";
	public static final String	DC_TITLE_QUERY				= "//*[name()='dc:title']";
	public static final String	INT_ID_QUERY				= METS_NAMESPACE
																	+ "/m:mets/@OBJID";
	public static final String	EXT_ID_QUERY				= METS_NAMESPACE
																	+ LMER_OBJECT_NAMESPACE
																	+ "/m:mets/m:amdSec/m:techMD/m:mdWrap/m:xmlData/lo:persistentIdentifier/text()";
	public static final String	TECHMD_ID_QUERY				= METS_NAMESPACE
																	+ "/m:mets/m:amdSec/m:techMD/@ID";
	public static final String	MIGRATION_QUERY				= METS_NAMESPACE
																	+ LMER_OBJECT_NAMESPACE
																	+ "/m:mets/m:amdSec/m:digiprovMD/m:mdWrap/m:xmlData/lo:oldObjectIdentifier/text()";
	public static final String	MPTR_QUERY					= METS_NAMESPACE
																	+ "/m:mets/m:structMap[@TYPE='ASSET']/m:div/descendant::*[name()='mptr']/@href";
	public static final String	FILES_WITHIN_FILEGRP_QUERY	= METS_NAMESPACE
																	+ "/m:mets/m:fileSec/m:fileGrp/m:file";
	public static final String	ANCESTOR_TECH_ID_QUERY		= "ancestor::*[name()='techMD']";
	public static final String	FILEGRP_FILE_TECHID_QUERY	= METS_NAMESPACE
																	+ "/m:mets/m:fileSec/m:fileGrp/m:file/@ADMID";
	public static final String	FILEGRP_FILEID_QUERY		= METS_NAMESPACE
																	+ "/m:mets/m:fileSec/m:fileGrp/m:file/@ID";
	public static final String	FILEURI_QUERY				= METS_NAMESPACE
																	+ "/m:mets/m:fileSec/m:fileGrp/m:file/m:FLocat/@href";
	public static final String	FILE_LINKEDTO_QUERY			= "//*[name()='lmerFile:linkedTo']";
	public static final String	JHOVE_FILE_FORMAT_QUERY		= METS_NAMESPACE
																	+ LMER_FILE_NAMESPACE
																	+ JHOVE_NAMESPACE
																	+ "/m:mets/m:amdSec/m:techMD/m:mdWrap/m:xmlData/lf:xmlData/jv:jhove/"
																	+ "jv:repInfo/jv:format/text()";

	// STATE (Instance variables) *****************************************

	// the query is an xpath-expression
	private String				characterisationQuery		= null;
	private String				characterisationValue		= null;

	// MANIPULATION (Manipulation - what the object does) *****************

	/***************************************************************************
	 * Constructor with criteria.
	 * 
	 * @param charQuery
	 * @param charValue
	 **************************************************************************/
	public ObjCharacter(Criteria crit) {
		super();
		this.characterisationQuery = crit.getXpath();
		this.characterisationValue = crit.getResult();
	}

	/***************************************************************************
	 * Constructor with criteria.
	 * 
	 * TODO the migration manager config file schemas should reference the same
	 * criteria
	 * 
	 * @param charQuery
	 * @param charValue
	 **************************************************************************/
	public ObjCharacter(
			koLibRIMigrationServiceRegistry.CriteriaDocument.Criteria crit) {
		super();
		this.characterisationQuery = crit.getXpath();
		this.characterisationValue = crit.getResult();
	}

	/***************************************************************************
	 * Constructor with object character field and value.
	 * 
	 * @param charQuery
	 * @param charValue
	 **************************************************************************/
	public ObjCharacter(String charQuery, String charValue) {
		super();
		this.characterisationQuery = charQuery;
		this.characterisationValue = charValue;
	}

	/***************************************************************************
	 * Convenience constructor for all results of this query.
	 * 
	 * @param charQuery
	 **************************************************************************/
	public ObjCharacter(String charQuery) {
		super();
		this.characterisationQuery = charQuery;
		this.characterisationValue = null;
	}

	/***************************************************************************
	 * Constructor which leaves the characterization query and value to set.
	 * 
	 **************************************************************************/
	public ObjCharacter() {
		super();
	}

	/***************************************************************************
	 * Returns the characterization query.
	 * 
	 * @return
	 **************************************************************************/
	public String getCharacterisationQuery() {
		return this.characterisationQuery;
	}

	/***************************************************************************
	 * Sets the characterization query.
	 * 
	 * @param characterisationQuery
	 **************************************************************************/
	public void setCharacterisationQuery(String characterisationQuery) {
		this.characterisationQuery = characterisationQuery;
	}

	/***************************************************************************
	 * Returns the characterization value.
	 * 
	 * @return
	 **************************************************************************/
	public String getCharacterisationValue() {
		return this.characterisationValue;
	}

	/***************************************************************************
	 * Sets the characterization value.
	 * 
	 * @param characterisationValue
	 **************************************************************************/
	public void setCharacterisationValue(String characterisationValue) {
		this.characterisationValue = characterisationValue;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return this.characterisationQuery + " == " + this.characterisationValue;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (obj.getClass().equals(ObjCharacter.class)) {
			if (this.characterisationQuery
					.equalsIgnoreCase(((ObjCharacter) obj)
							.getCharacterisationQuery())
					&& this.characterisationValue
							.equalsIgnoreCase(((ObjCharacter) obj)
									.getCharacterisationValue())) {
				return true;
			}

			return false;
		}

		return super.equals(obj);
	}

}
