/*******************************************************************************
 * de.langzeitarchivierung.kopal.processstarter.migrationmanager / ObjCharacterTranslator.java
 * 
 * Copyright 2005-2007 by Project kopal
 * 
 * http://kopal.langzeitarchivierung.de/
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307, USA
 *
 *
 *******************************************************************************
 *
 *
 * CHANGELOG:
 * 10.07.2007	koLibRI version 1.0
 ******************************************************************************/

package de.langzeitarchivierung.kopal.processstarter.migrationmanager;

import java.util.List;
import java.util.Set;

import org.apache.xmlbeans.XmlException;

/*******************************************************************************
 * Interface for translating object characterisation to the corresponding data
 * store.
 * 
 * TODO How can we characterise embedded formats?
 * 
 * @author Ludwig
 * @version 0.5
 ******************************************************************************/

public interface ObjCharacterTranslator
{

	/***************************************************************************
	 * Returns all internal ids of the characterized objects
	 * 
	 * @param objCharacter
	 * @return the set of internal ids
	 * @throws UnsupportedOperationException If the object characterization
	 *             requires to query a data field which is not supported by the
	 *             data management or translator class.
	 **************************************************************************/
	public Set<String> getCharacterisedObjects(ObjCharacter objCharacter)
			throws Exception;

	/***************************************************************************
	 * Returns the value of the requested property of the requested object.
	 * 
	 * @param internalId the internal id of the object to query. If it is null
	 *            or empty all available objects are queried.
	 * @param objCharacter The object property to query. The characterisation
	 *            value is ignored.
	 * @return the values of the requested object property. This may be more
	 *         than one.
	 * @throws UnsupportedOperationException If this property cannot be queried
	 *             in the implemented data managemetn
	 **************************************************************************/
	public List<String> getCharacterisedObjectValue(String internalId,
			ObjCharacter objCharacter) throws Exception;

	/***************************************************************************
	 * Checks if an object has no successor.
	 * 
	 * @param internalId The internal id of the object to check.
	 * @return true if the object with the specified internal id has no
	 *         successor, otherwise false.
	 **************************************************************************/
	public boolean isNotMigrated(String internalId) throws Exception;

}
