/*******************************************************************************
 * de.langzeitarchivierung.kopal.processstarter.migrationmanager / ObjectDependencyAnalysator.java
 * 
 * Copyright 2005-2007 by Project kopal
 * 
 * http://kopal.langzeitarchivierung.de/
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307, USA
 *
 *
 *******************************************************************************
 *
 *
 * CHANGELOG:
 * 10.07.2007	koLibRI version 1.0
 ******************************************************************************/

package de.langzeitarchivierung.kopal.processstarter.migrationmanager;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.xmlbeans.XmlException;

/*******************************************************************************
 * Analyses the dependencies between objects.
 * 
 * @author ludwig
 * @version 0.5
 ******************************************************************************/

public class ObjectDependencyAnalysator
{

	// STATE (Instance variables) *****************************************

	ObjCharacterTranslator	objChrTranslator	= null;

	// MANIPULATION (Manipulation - what the object does) *****************

	/***************************************************************************
	 * Returns a list of internal ids of the objects on which the object with
	 * specified internal id depends.
	 * 
	 * @param internalId
	 * @return
	 **************************************************************************/
	public List<String> getDependentObjects(String internalId) throws Exception {
		List<String> dependentObjects = new ArrayList<String>();

		dependentObjects = getRecursiveDependentObjects(internalId,
				dependentObjects);

		// TODO apply additional checks?

		return dependentObjects;
	}

	/***************************************************************************
	 * Sets the object character translator to use.
	 * 
	 * TODO allow more than one object character translator because some may not
	 * support the necessary data fields to analyse the object dependencies?
	 * 
	 * @param objChrTranslator
	 **************************************************************************/
	public void setObjChrTranslator(ObjCharacterTranslator objChrTranslator) {
		this.objChrTranslator = objChrTranslator;
	}

	// INTERNAL (Internal - implementation details, local classes, ...) **

	/***************************************************************************
	 * Returns all internal ids of the objects directly or indirectly depending
	 * on this one.
	 * 
	 * @param internalId
	 * @param dependentObjects
	 * @return
	 **************************************************************************/
	private List<String> getRecursiveDependentObjects(String internalId,
			List<String> dependentObjects) throws Exception {

		List<String> mptrObjects = getMptrs(internalId);

		for (Iterator mptrIter = mptrObjects.iterator(); mptrIter.hasNext();) {
			String mptr = (String) mptrIter.next();
			if (!dependentObjects.contains(mptr)) {
				dependentObjects.add(mptr);
				dependentObjects.addAll(getRecursiveDependentObjects(mptr,
						dependentObjects));
			}
		}

		return dependentObjects;
	}

	/***************************************************************************
	 * Returns a list of the mets pointers (mptr) in the file section of the
	 * object with the specified internal id.
	 * 
	 * @param internalId
	 * @return
	 **************************************************************************/
	private List<String> getMptrs(String internalId) throws Exception {
		return this.objChrTranslator.getCharacterisedObjectValue(internalId,
				new ObjCharacter(ObjCharacter.MPTR_QUERY));
	}

}
