/*******************************************************************************
 * de.langzeitarchivierung.kopal.ui / DBMessageListener.java
 *  
 * Copyright 2005-2007 by Project kopal
 * 
 * http://kopal.langzeitarchivierung.de/
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307, USA
 * 
 * 
 *******************************************************************************
 *
 *
 * CHANGELOG:
 * 10.07.2007	koLibRI version 1.0
 ******************************************************************************/

package de.langzeitarchivierung.kopal.ui;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import de.langzeitarchivierung.kopal.ProcessData;
import de.langzeitarchivierung.kopal.Status;
import de.langzeitarchivierung.kopal.Step;
import de.langzeitarchivierung.kopal.util.Configurator;
import de.langzeitarchivierung.kopal.util.db.MappToHib;

/*******************************************************************************
 * ErrorMessageListener which logs all status events of the types ERROR, WARNING
 * and CANCELED to the database
 * 
 * @author Ludwig
 * @version 20060215
 * @since 20060215
 ******************************************************************************/

public class DBMessageListener implements StatusListener
{

	// STATE (Instance variables) *****************************************

	ProcessData					proc;

	// CLASS VARIABLES (Static variables) ***********************************

	private static Logger		defaultLogger			= Logger
																.getLogger("de.langzeitarchivierung.kopal");
	private static List<String>	logMessagesWithStatus	= new ArrayList<String>();

	// CREATION (Constructors, factory methods, static/inst init) **********

	/***************************************************************************
	 * Default Constructor.
	 **************************************************************************/
	public DBMessageListener() throws Exception {
		super();
		Configurator.configure(this);
	}

	/***************************************************************************
	 * @param _proc
	 * @throws NoSuchMethodException
	 * @throws IllegalAccessException
	 * @throws InvocationTargetException
	 **************************************************************************/
	public DBMessageListener(ProcessData _proc) throws Exception {
		super();
		this.proc = _proc;
		Configurator.configure(this);
	}

	/***************************************************************************
	 * @see de.langzeitarchivierung.kopal.ui .StatusListener#update(
	 *      de.langzeitarchivierung.kopal.ui.StatusChangedEvent)
	 **************************************************************************/
	public void update(StatusChangedEvent sc) {
		Step stp = (Step) sc.getSource();
		Status changedStatus = stp.getStatus();

		// Log all data to the database as configured in the main config file.
		if (logMessagesWithStatus.contains(changedStatus.getStatusText())) {
			try {
				// Add the event to the database.
				MappToHib.addEvent(stp.getProcessData(), stp, System
						.currentTimeMillis());
			}
			catch (Exception e) {
				defaultLogger.log(Level.SEVERE, "Could not log event to "
						+ "database: '" + changedStatus.getDescription() + "'");
				e.printStackTrace();
			}
		}
	}

	// GET & SET METHODS **************************************************

	/***************************************************************************
	 * @param element
	 **************************************************************************/
	public static void setLogMessagesWithStatus(String element) {
		if (!logMessagesWithStatus.contains(element)) {
			logMessagesWithStatus.add(element);
		}
	}

}
