/*******************************************************************************
 * de.langzeitarchivierung.kopal.util / FormatInfo.java
 *  
 * Copyright 2005-2007 by Project kopal
 * 
 * http://kopal.langzeitarchivierung.de/
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307, USA
 * 
 * 
 *******************************************************************************
 *
 *
 * CHANGELOG:
 * 10.07.2007	koLibRI version 1.0
 ******************************************************************************/

package de.langzeitarchivierung.kopal.util;

import java.io.Serializable;

/*******************************************************************************
 * A datatype for format specific information.
 * 
 * @author Matthias Neubauer, German National Library
 * @version 20070125
 * @see de.langzeitarchivierung.kopal.util.FormatRegistry
 * @since 20050825
 ******************************************************************************/

public class FormatInfo implements Serializable
{

	// CLASS VARIABLES (Static variables) ******************************

	private static final long	serialVersionUID			= 1L;

	// STATE (Instance variables) *****************************************

	private String				mimeType					= "";
	private String				fileType					= "";
	private String				fileTypeVersion				= "";
	private String				fileTypeCategory			= "";
	private String				fileTypeID					= "";
	private String				fileTypeStatus				= "";
	private String				fileTypeViewerApplication	= "";
	private String				fileExtension				= "";

	/***************************************************************************
	 * @see java.lang.Object#toString()
	 **************************************************************************/
	@Override
	public String toString() {
		String result = "";

		result = "fileTypeCategory: " + this.fileTypeCategory;
		result += "\nfileType: " + this.fileType;
		result += "\nfileTypeID: " + this.fileTypeID;
		result += "\nfileTypeStatus: " + this.fileTypeStatus;
		result += "\nfleTypeVersion: " + this.fileTypeViewerApplication;
		result += "\nfileTypeViewerApplication: "
				+ this.fileTypeViewerApplication;
		result += "\nmimeType: " + this.mimeType;

		return result;
	}

	// GET & SET METHODS **********************************************

	/***************************************************************************
	 * Get the category of this format.
	 * 
	 * @return The media category of this format
	 **************************************************************************/
	public String getFileTypeCategory() {
		return this.fileTypeCategory;
	}

	/***************************************************************************
	 * Get the name of this format.
	 * 
	 * @return The JHOVE name of this format
	 **************************************************************************/
	public String getFileType() {
		return this.fileType;
	}

	/***************************************************************************
	 * Get the mimetype of this format.
	 * 
	 * @return The mimetype of this format
	 **************************************************************************/
	public String getMimeType() {

		System.out.println("getMimeType()");

		return this.mimeType;
	}

	/***************************************************************************
	 * Get the default viewer application of this format.
	 * 
	 * @return The name of an application, able to view the file
	 **************************************************************************/
	public String getFileTypeViewerApplication() {
		return this.fileTypeViewerApplication;
	}

	/***************************************************************************
	 * Get the version of this format.
	 * 
	 * @return The version of the specific format
	 **************************************************************************/
	public String getFileTypeVersion() {
		return this.fileTypeVersion;
	}

	/***************************************************************************
	 * Get the state of this format.
	 * 
	 * @return The state of the specific format-version. Either "Active" or
	 *         "Obsolete"
	 **************************************************************************/
	public String getFileTypeStatus() {
		return this.fileTypeStatus;
	}

	/***************************************************************************
	 * Get the unique ID of this format.
	 * 
	 * @return The unique id of the specific format version
	 **************************************************************************/
	public String getFileTypeID() {
		return this.fileTypeID;
	}

	/***************************************************************************
	 * Get the default file extension of this format.
	 * 
	 * @return The standard file extension of this type
	 **************************************************************************/
	public String getFileExtension() {
		return this.fileExtension;
	}

	/***************************************************************************
	 * Set the category of this format.
	 * 
	 * @param _fileTypeCategory The category of this format
	 **************************************************************************/
	public void setFileTypeCategory(String _fileTypeCategory) {
		this.fileTypeCategory = _fileTypeCategory;
	}

	/***************************************************************************
	 * Set the default viewer application of this format.
	 * 
	 * @param _fileTypeViewerApplication The default viewer application
	 **************************************************************************/
	public void setFileTypeViewerApplication(String _fileTypeViewerApplication) {
		this.fileTypeViewerApplication = _fileTypeViewerApplication;
	}

	/***************************************************************************
	 * Set the default file extension of this format.
	 * 
	 * @param _fileExtension The default file extension
	 **************************************************************************/
	public void setFileExtension(String _fileExtension) {
		this.fileExtension = _fileExtension;
	}

	/***************************************************************************
	 * Set the name of this format.
	 * 
	 * @param _fileType The formats name
	 **************************************************************************/
	public void setFileType(String _fileType) {
		this.fileType = _fileType;
	}

	/***************************************************************************
	 * Set the unique ID of this format.
	 * 
	 * @param _fileTypeID The unique ID for this format
	 **************************************************************************/
	public void setFileTypeID(String _fileTypeID) {
		this.fileTypeID = _fileTypeID;
	}

	/***************************************************************************
	 * Set the mimetype of this format.
	 * 
	 * @param _mimeType The mimetype of this format
	 **************************************************************************/
	public void setMimeType(String _mimeType) {
		this.mimeType = _mimeType;
	}

	/***************************************************************************
	 * Set the state of this format.
	 * 
	 * @param _fileTypeStatus The State of this format, either "Active" or
	 *            "Obsolete"
	 **************************************************************************/
	public void setFileTypeStatus(String _fileTypeStatus) {
		this.fileTypeStatus = _fileTypeStatus;
	}

	/***************************************************************************
	 * Set the version of this format.
	 * 
	 * @param _fileTypeVersion The version number
	 **************************************************************************/
	public void setFileTypeVersion(String _fileTypeVersion) {
		this.fileTypeVersion = _fileTypeVersion;
	}

}
