/*******************************************************************************
 * de.langzeitarchivierung.kopal.util / FormatMapper.java
 *  
 * Copyright 2005-2007 by Project kopal
 * 
 * http://kopal.langzeitarchivierung.de/
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307, USA
 * 
 * 
 *******************************************************************************
 * 
 * 
 * CHANGELOG:
 * 10.07.2007	koLibRI version 1.0
 ******************************************************************************/

package de.langzeitarchivierung.kopal.util;

/*******************************************************************************
 * A class that maps a format registry in DIAS format to the names and versions
 * used by JHOVE from HUL.
 * 
 * @author Matthias Neubauer, German National Library
 * @version 20070213
 * @see de.langzeitarchivierung.kopal.util.FormatRegistry
 * @since 20060102
 ******************************************************************************/

public class FormatMapper
{

	// MANIPULATION (Manipulation - what the object does) ***************

	/***************************************************************************
	 * A method that maps one format registry to another.
	 * 
	 * @author Matthias Neubauer, German National Library
	 * @see de.langzeitarchivierung.kopal.util.FormatRegistry
	 * @param formatreg - The ArrayList that contains the format registry to be
	 *            mapped
	 * @param mappingFile - Path to the xml format mapping file
	 * @return true - if the mapping was successful
	 **************************************************************************/
	public boolean map(java.util.ArrayList formatreg, String mappingFile) {
		String xmlstring = "";
		java.io.File xmlfile = new java.io.File(mappingFile);

		if (xmlfile.exists()) {
			// Use the backup file.
			xmlstring = FileUtils.readFile(xmlfile);
		}

		// Get the single "Item" Nodes.
		org.w3c.dom.Document doc = KopalXMLParser
				.getDocumentFromString(xmlstring);

		org.w3c.dom.NodeList formats = doc.getElementsByTagName("map");

		FormatInfo tempform = new FormatInfo();

		// Go through the List...
		for (int i = 0; i < formatreg.size(); i++) {
			tempform = (FormatInfo) formatreg.get(i);
			int numberOfItems = formats.getLength();

			// Go through the List of entries an create a FormtaInfo for each.
			for (int j = 0; j < numberOfItems; j++) {
				String oldname = KopalXMLParser.getDirectSubNodes(
						formats.item(j), "oldname").item(0).getFirstChild()
						.getNodeValue();

				if (tempform.getFileType().equals(oldname)) {
					String newname = KopalXMLParser.getDirectSubNodes(
							formats.item(j), "newname").item(0).getFirstChild()
							.getNodeValue();
					String version = "";
					boolean addoldversion = false;

					try {
						version = KopalXMLParser.getDirectSubNodes(
								formats.item(j), "newversion").item(0)
								.getFirstChild().getNodeValue();
						if (KopalXMLParser.getDirectSubNodes(formats.item(j),
								"newversion").item(0).getAttributes()
								.getNamedItem("addoldversion").getNodeValue()
								.equals("true")) {
							addoldversion = true;
						}
					}
					catch (java.lang.ArrayIndexOutOfBoundsException e) {
						// Fall through.
					}

					tempform.setFileType(newname);

					if (!version.equals("") && version != null) {
						if (addoldversion) {
							version += tempform.getFileTypeVersion();
						}
						tempform.setFileTypeVersion(version);
					}

					break;
				}
			}
		}

		return true;
	}

}
