/*******************************************************************************
 * de.langzeitarchivierung.kopal.util / KopalConsoleLogFormatter.java
 * 
 * Copyright 2005-2007 by Project kopal
 * 
 * http://kopal.langzeitarchivierung.de/
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307, USA
 *
 * 
 ******************************************************************************
 * 
 * 
 * CHANGELOG:
 * 10.07.2007	koLibRI version 1.0
 ******************************************************************************/

package de.langzeitarchivierung.kopal.util;

import java.util.logging.Level;
import java.util.logging.LogRecord;

/*******************************************************************************
 * The KopalConsoleLogFormatter formats the kopal console log.
 * 
 * @author Funk
 * @version 20061204
 * @since 20060125
 ******************************************************************************/

public class KopalConsoleLogFormatter extends java.util.logging.Formatter
{

	// CLASS VARIABLES (Static variables) *********************************

	private int		oldThreadId		= 0;
	private String	oldClassName	= null;

	// CREATION (Constructors, factory methods, static/inst init) **********

	/***************************************************************************
	 * Default constructor
	 **************************************************************************/
	public KopalConsoleLogFormatter() {
		super();
	}

	// MANIPULATION (Manipulation - what the object does) ******************

	/***************************************************************************
	 * @see java.util.logging.Formatter#format(java.util.logging.LogRecord)
	 **************************************************************************/
	@Override
	public String format(LogRecord record) {
		String result = "";

		String className = record.getSourceClassName();
		String methodName = record.getSourceMethodName();
		int threadId = record.getThreadID();
		String recordMessage = record.getMessage();

		if (this.oldClassName == null) {
			this.oldClassName = "";
		}

		// Show thread ID and class Name if different from last ID.
		if (this.oldThreadId != threadId) {
			result += "\n[#" + threadId + "]\t" + className + "." + methodName
					+ "()\n";
		}
		// Only show the class name if thread ID has not changed since last log
		// entry.
		else if (!this.oldClassName.equals(className)) {
			result += "\n\t" + className + "." + methodName + "()\n";
		}

		// Set the old values.
		this.oldThreadId = threadId;
		this.oldClassName = className;

		// If the log level is equal to WARNING or equal to SEVERE, add it to
		// the log message if not occuring in an action module.
		Level level = record.getLevel();
		String levelMessage = "";

		if (!className.equals("de.langzeitarchivierung."
				+ "kopal.ui.LoggingStatusListener")) {
			// Put some spaces behind the level.
			for (int i = 0; i < (9 - level.toString().length()); i++) {
				levelMessage += " ";
			}

			levelMessage = "[" + level + "]" + levelMessage;
		}

		result += "\t" + levelMessage + recordMessage + "\n";

		return result;
	}

}
