/*******************************************************************************
 * de.langzeitarchivierung.kopal.util / KopalTimestamp.java
 * 
 * Copyright 2005-2007 by Project kopal
 * 
 * http://kopal.langzeitarchivierung.de/
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307, USA
 *
 * 
 ******************************************************************************
 * 
 * 
 * CHANGELOG:
 * 10.07.2007	koLibRI version 1.0
 ******************************************************************************/

package de.langzeitarchivierung.kopal.util;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.GregorianCalendar;
import java.text.DecimalFormat;

/*******************************************************************************
 * Class for generating timestamps.
 * 
 * @author Funk, Ludwig
 * @version 20060123
 ******************************************************************************/
public class KopalTimestamp
{

	/***************************************************************************
	 * W3C compliant timestamp: "yyyy-MM-dd'T'HH:mm:ss"
	 * 
	 * @return "yyyy-MM-dd'T'HH:mm:ss"
	 **************************************************************************/
	public static String getTimestamp() {
		SimpleDateFormat dateFormat = new SimpleDateFormat(
				"yyyy-MM-dd'T'HH:mm:ss");
		SimpleDateFormat timezone = new SimpleDateFormat("Z");
		GregorianCalendar timeStamp = new GregorianCalendar();
		return dateFormat.format(timeStamp.getTime())
				+ timezone.format(timeStamp.getTime()).substring(0, 3) + ":"
				+ timezone.format(timeStamp.getTime()).substring(3);
	}

	/***************************************************************************
	 * Timestamp without timezone: "yyyy-MM-dd'T'HH:mm:ss"
	 * 
	 * @param milliseconds
	 * @return "yyyy-MM-dd'T'HH:mm:ss"
	 **************************************************************************/
	public static String getTimestamp(long milliseconds) {
		SimpleDateFormat dateFormat = new SimpleDateFormat(
				"yyyy-MM-dd'T'HH:mm:ss");
		GregorianCalendar timeStamp = new GregorianCalendar();
		timeStamp.setTimeInMillis(milliseconds);
		return dateFormat.format(timeStamp.getTime());
	}

	/***************************************************************************
	 * Parses a date string as delivered by dias and returns its represenation
	 * in milliseconds since the begin of the epoch.
	 * 
	 * @param diasDate a date string as delivered by dias
	 * @return timestamp in milliseconds
	 * @throws ParseException
	 **************************************************************************/
	public static long parseTimestamp(String diasDate) throws ParseException {
		SimpleDateFormat dateReader = new SimpleDateFormat(
				"yyyy-MM-dd'T'HH:mm:ss.SSS");
		return dateReader.parse(diasDate).getTime();
	}

	/***************************************************************************
	 * Timestamp without colons.
	 * 
	 * @return "yyyy-MM-dd'T'HH_mm_ss"
	 **************************************************************************/
	public static String getFilenameTimestamp() {
		SimpleDateFormat dateFormat = new SimpleDateFormat(
				"yyyy-MM-dd'T'HH_mm_ss");
		SimpleDateFormat timezone = new SimpleDateFormat("Z");
		GregorianCalendar timeStamp = new GregorianCalendar();
		return dateFormat.format(timeStamp.getTime())
				+ timezone.format(timeStamp.getTime()).substring(0, 3) + "_"
				+ timezone.format(timeStamp.getTime()).substring(3);
	}

	/***************************************************************************
	 * Replace all occurences of non ascii letters or digits with "_".
	 * 
	 * TODO Well, the method should also replace some letters like ���...
	 * 
	 * @param source
	 * @return the modified string
	 **************************************************************************/
	public static String makeFilenameCompatible(String source) {
		StringBuffer result = null;
		for (int i = 0; i < source.length(); i++) {
			if (!Character.isLetterOrDigit(source.charAt(i))) {
				result.append(source.charAt(i));
			}
			else {
				result.append("_");
			}
		}
		return result.toString();
	}

	/***************************************************************************
	 * W3C compliant timestamp with milliseconds: "yyyy-MM-dd'T'HH:mm:ss.S"
	 * 
	 * @return "yyyy-MM-dd'T'HH:mm:ss.SSS"
	 **************************************************************************/
	public static String getTimestampWithMs() {
		SimpleDateFormat dateFormat = new SimpleDateFormat(
				"yyyy-MM-dd'T'HH:mm:ss.SSS");
		SimpleDateFormat timezone = new SimpleDateFormat("Z");
		GregorianCalendar timeStamp = new GregorianCalendar();
		return dateFormat.format(timeStamp.getTime())
				+ timezone.format(timeStamp.getTime()).substring(0, 3) + ":"
				+ timezone.format(timeStamp.getTime()).substring(3);
	}

	/***************************************************************************
	 * W3C compliant timestamp with milliseconds: "yyyy-MM-dd'T'HH:mm:ss.S"
	 * 
	 * @param milliseconds - the milliseconds since 1970 for which a timestamp
	 *            will be computed
	 * @return "yyyy-MM-dd'T'HH:mm:ss.SSS"
	 **************************************************************************/
	public static String getTimestampWithMs(long milliseconds) {
		SimpleDateFormat dateFormat = new SimpleDateFormat(
				"yyyy-MM-dd'T'HH:mm:ss.SSS");
		SimpleDateFormat timezone = new SimpleDateFormat("Z");
		GregorianCalendar timeStamp = new GregorianCalendar();
		timeStamp.setTimeInMillis(milliseconds);
		return dateFormat.format(timeStamp.getTime())
				+ timezone.format(timeStamp.getTime()).substring(0, 3) + ":"
				+ timezone.format(timeStamp.getTime()).substring(3);
	}

	/***************************************************************************
	 * Timestamp with milliseconds without colons.
	 * 
	 * @return "yyyy-MM-dd'T'HH_mm_ss.SSS"
	 **************************************************************************/
	public static String getFilenameTimestampWithMs() {
		SimpleDateFormat dateFormat = new SimpleDateFormat(
				"yyyy-MM-dd'T'HH_mm_ss_SSS");
		SimpleDateFormat timezone = new SimpleDateFormat("Z");
		GregorianCalendar timeStamp = new GregorianCalendar();
		return dateFormat.format(timeStamp.getTime())
				+ timezone.format(timeStamp.getTime()).substring(0, 3) + "_"
				+ timezone.format(timeStamp.getTime()).substring(3);
	}

	/***************************************************************************
	 * Return the duration without milliseconds per default and use some default
	 * unit strings.
	 * 
	 * @param duration
	 * @return
	 **************************************************************************/
	public static String getDurationInHours(long duration) {
		return getDurationInHours(duration, false, "hour", "minute", "second",
				"millisecond");
	}

	/***************************************************************************
	 * Interpret and format a long as a duration. Useful for a perfomance check
	 * with two System.currentTimeMillis().
	 * 
	 * @param duration The time to display the duration for in milliseconds.
	 * @param showMils Show the milliseconds or hide them.
	 * @param disp_hor The hour display string.
	 * @param disp_min The minute display string.
	 * @param disp_sec The second display string.
	 * @param disp_mil The millisecond display string.
	 * @return String The duration in hours.
	 **************************************************************************/
	public static String getDurationInHours(long duration, boolean showMils,
			String disp_hor, String disp_min, String disp_sec, String disp_mil) {
		String result = "";
		DecimalFormat df = new DecimalFormat("0");

		long hors = (duration / 3600000); // get hours
		long rest = (duration % 3600000); // get rest

		long mins = (rest / 60000); // get minutes
		rest = (rest % 60000); // get rest

		long secs = (rest / 1000); // get seconds
		rest = (rest % 1000); // get rest

		long mils = rest;

		if (hors > 0) {
			result += df.format(hors) + " " + disp_hor
					+ (hors > 1 ? "s " : " ");
		}
		if (mins > 0) {
			result += df.format(mins) + " " + disp_min
					+ (mins > 1 ? "s " : " ");
		}
		if (secs > 0) {
			result += df.format(secs) + " " + disp_sec
					+ (secs > 1 ? "s " : " ");
		}
		if (mils > 0 && showMils) {
			result = df.format(duration) + " " + disp_mil
					+ (mils > 1 ? "s " : " ");
		}
		if (result.equals("")) {
			result = "less than a " + disp_sec;
		}

		return result.trim();
	}

}
