/*******************************************************************************
 * de.langzeitarchivierung.kopal.util / WSUtils.java
 *  
 * Copyright 2005-2007 by Project kopal
 * 
 * http://kopal.langzeitarchivierung.de/
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307, USA
 *
 * 
 ******************************************************************************
 * 
 * 
 * CHANGELOG:
 * 10.07.2007	koLibRI version 1.0
 ******************************************************************************/

package de.langzeitarchivierung.kopal.util;

import de.langzeitarchivierung.kopal.retrieval.AccessLayer;
import de.langzeitarchivierung.kopal.retrieval.DiasAccessResponse;
import de.langzeitarchivierung.kopal.ws.wsdl2java.ResponseElemDocument;
import de.langzeitarchivierung.kopal.ws.wsdl2java.UserType;

/*******************************************************************************
 * Class for utilities around Access functionality
 * 
 * @author Kadir Karaca Koçer, German National Library
 * @version 20061220
 * @see AccessLayer
 * @see DiasAccessResponse
 * @since 20061103
 ******************************************************************************/

public class WSUtils
{

	// CONSTANTS **********************************************************

	/***************************************************************************
	 * Path to configuration file
	 **************************************************************************/
	public static String CONFIG_FILE = "c:/temp/config.xml";

	// MANIPULATION (Manipulation - what the object does) ******************

	/***************************************************************************
	 * Creates a default ResponseElemDocument object.
	 * 
	 * @return A new ResponseElemDocument with default values.
	 * @author Kadir Karaca Koçer, German National Library
	 * @version 20070607
	 * @see ResponseElemDocument
	 * @since 20070615
	 **************************************************************************/
	public static ResponseElemDocument createDefaultResponse() {
		ResponseElemDocument response = ResponseElemDocument.Factory.newInstance();
		ResponseElemDocument.ResponseElem re = ResponseElemDocument.ResponseElem.Factory.newInstance();

		re.setStatusCode(598);
		re.setResponseText("Sorry. This service is not implemented at the moment.");
		re.setDiasId("Not used in this function.");
		re.setLinkToFile("Not used in this function.");
		re.setWaitTime(0);
		re.setErrorMessage("No extra error message.");
		re.setExtraInfo("No extra info");
		// response.getResponseElem().setCustomdata("No custom data");

		response.setResponseElem(re);
		return response;
	}

	/***************************************************************************
	 * Checks if the user has the nedded privileges and correct password.
	 * Returns 0 if everything is ok.
	 * 
	 * @param user User
	 * @param action Action to check for if the processing allowed
	 * @return An integer as status code. 0 --> allowed, otherweise some error
	 * @author Kadir Karaca Koçer, German National Library
	 * @version 20070620
	 * @see WSRetrieval
	 * @since 20061103
	 **************************************************************************/
	public static int checkRights(UserType user, String action) {
		// TODO: implement a "REAL" management to check user rights
		// 0 --> Action allowed.
		// Unknown User
		// Wrong Password
		// Account deactivated
		// Action not allowed for this user

		// FAKE!!
		System.out.println(user.getUserName() + "\t" + action);
		return 0;
	}

	/***************************************************************************
	 * Generates matching ResponseElemDocumentImpl for a given autorisation
	 * error
	 * 
	 * @param code Error code
	 * @return Response object
	 * @author Kadir Karaca Koçer, German National Library
	 * @version 20061220
	 * @see ResponseElemDocument
	 * @since 20061103
	 **************************************************************************/
	public static ResponseElemDocument authorisationError(int code) {
		ResponseElemDocument response = createDefaultResponse();

		// TODO: implement a "REAL" error codes generation
		response.getResponseElem().setStatusCode(599);
		response.getResponseElem().setResponseText(
				"Authorisation Failed! Code = " + code);
		return response;
	}

	/***************************************************************************
	 * Changes the password of the given user
	 * 
	 * @param user User to change the password
	 * @param newPassword New Password
	 * @return Response object
	 * @author Kadir Karaca Koçer, German National Library
	 * @version 20070620
	 * @see ResponseElemDocument
	 * @since 20070620
	 **************************************************************************/
	public static ResponseElemDocument changePassword(UserType user,
			String newPassword) {
		checkRights(user, "changePassword");

		// TODO: implement a user autentification system
		System.out.println(newPassword);

		return createDefaultResponse();
	}

	/***************************************************************************
	 * Returns the string representation of relativ path using the DIAS internal
	 * ID. (See the DIP Specification published by IBM Holland)
	 * 
	 * @param intID DIAS intern unique ID
	 * @return Generated path to this object in cache
	 * @author Kadir Karaca Koçer, German National Library
	 * @version 20061220
	 * @see WSResponse
	 * @since 20061103
	 **************************************************************************/
	public static String internalIdToDir(String intID) {
		String filteredID = intID
				.substring(intID.length() - 22, intID.length());
		int length = filteredID.length();
		String dirs = "/" + filteredID.charAt(length - 7) + "/"
				+ filteredID.charAt(length - 8) + "/"
				+ filteredID.charAt(length - 10);

		dirs = dirs + "/" + filteredID + "/";
		System.out.println("\nTemporary directory: " + dirs);
		return dirs;
	}

	/***************************************************************************
	 * Returns the string array representation of relativ path using the DIAS
	 * internal ID. (See the DIP Specification published by IBM Holland)
	 * 
	 * @param intID DIAS intern unique ID
	 * @return Generated path to this object in cache. Each directory level + 4.
	 *         the whole
	 * @author Kadir Karaca Koçer, German National Library
	 * @version 20061220
	 * @see WSResponse
	 * @since 20061103
	 **************************************************************************/
	public static String[] internalIdToDirs(String intID) {
		String[] dirs = { "", "", "", "" };
		dirs[3] = intID.substring(intID.length() - 22, intID.length());
		int length = dirs[3].length();
		dirs[0] = "/" + dirs[3].charAt(length - 7) + "/";
		dirs[1] = dirs[3].charAt(length - 8) + "/";
		dirs[2] = dirs[3].charAt(length - 10) + "/";
		return dirs;
	}

	/***************************************************************************
	 * Check if a given file name exists. If yes actualize the timestamp of the
	 * directory its in
	 * 
	 * @param name The complete name of the file
	 * @return true if the file exists, false otherweise
	 **************************************************************************/
	public static boolean fileAlreadyThere(String name) {
		try {
			// is file there?
			java.io.File file = new java.io.File(name);
			if (file.exists()) {
				file.getParentFile().setLastModified(System.currentTimeMillis());
				return true;
			}
			// else
			return false;
		}
		catch (Exception e) {
			return false;
		}
	}

}
