/*******************************************************************************
 * de.langzeitarchivierung.kopal.ws / TestWebServices.java
 * 
 * Copyright 2005-2007 by Project kopal
 * 
 * http://kopal.langzeitarchivierung.de/
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307, USA
 * 
 * 
 *******************************************************************************
 *
 *
 * CHANGELOG:
 * 12.07.2007	koLibRI version 1.0
 ******************************************************************************/

package de.langzeitarchivierung.kopal.ws;

import java.rmi.RemoteException;
import org.apache.axis2.AxisFault;
import org.apache.xmlbeans.XmlObject;

import de.langzeitarchivierung.kopal.ws.wsdl2java.*;

/*******************************************************************************
 * Class to test the functionality of implemented Web Services. It also
 * demonstrates how to call them
 * 
 * @author Kadir Karaca Koçer, German National Library
 * @version 20070621
 * @see WebServiceClient
 * @see WebServiceServer
 * @see KolibriWebServiceSkeletonInterface
 * @see ResponseElemDocument
 * @since 20070621
 ******************************************************************************/

public class TestWebServices
{
	private static String	lmer	= "<?xml version = \"1.0\" encoding = \"UTF-8\"?>\n"
											+ "<lmer xmlns=\"http://www.ddb.de/LMER\" xmlns:lmerObject=\"http://www.ddb.de/LMERobject\"\n"
											+ "xmlns:lmerProcess=\"http://www.ddb.de/LMERprocess\" xmlns:lmerFile=\"http://www.ddb.de/LMERfile\"\n"
											+ "xmlns:lmerModification=\"http://www.ddb.de/LMERmodification\"\n"
											+ "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n"
											+ "xsi:schemaLocation=\"http://www.ddb.de/LMER http://www.ddb.de/standards/lmer/lmer.xsd http://www.ddb.de/LMERobject http://www.ddb.de/standards/lmer/lmer-object.xsd http://www.ddb.de/LMERprocess http://www.ddb.de/standards/lmer/lmer-process.xsd http://www.ddb.de/LMERfile http://www.ddb.de/standards/lmer/lmer-file.xsd http://www.ddb.de/LMERmodification http://www.ddb.de/standards/lmer/lmer-modification.xsd\">\n"
											+ "<object ID=\"O12345\">"
											+ "<lmerObject:objectIdentifier>obj123</lmerObject:objectIdentifier>"
											+ "<lmerObject:persistentIdentifier>urn:nbn:de:gbv:104-opus-291</lmerObject:persistentIdentifier>"
											+ "<lmerObject:transferURL>ftp://ftp.bib-test.de/dnb/doc1234567890/</lmerObject:transferURL>"
											+ "<lmerObject:transferChecksum CHECKSUMTYPE=\"SHA-1\">304ac95579c21f3498dfdff7117d3845220d34f</lmerObject:transferChecksum>"
											+ "</object></lmer>";

	/***************************************************************************
	 * @param args
	 * @exception AxisFault
	 * @exception RemoteException
	 **************************************************************************/
	public static void main(String[] args) {
		// Test values
		String url = "http://localhost:8080/axis2/services/kolibriWebService";
		// String urn = "urn:nbn:de:1111-20070325-3-00548-2";
		String urn = "urn:nbn:de:1111-20070325-3-00543-7";
		// String intId ="urn:diasid:ast:kop01:0200704260149209930000";
		String intId = "urn:diasid:ast:kop01:0200704260154290930000";
		String file = "mets.xml";
		String rootDir = "c:/temp/cache";
		String policy = "dissertation";
		String linkToSIP = "";
		int timeLimit = 4;
		java.util.GregorianCalendar date = new java.util.GregorianCalendar();
		CustomXMLType[] metadata = new CustomXMLType[1];
		String diasRetrieverURL = "https://dias.gwdg.de/dias/Retriever";
		String diasLoaderURL = "";
		String fileFormatListURL = "file://c:/temp/fileformatlist.xml";
		String protocolRetrievalWS = "sftp://";
		String urlRetrievalWS = "localhost:8888/retrieval/";
		String tempDir = "d:/kolibri/temp/";

		// Create a client and send SOAP document/literals
		try {
			ResponseElemDocument response = ResponseElemDocument.Factory.newInstance();
			WebServiceClient ws = new WebServiceClient("Karaca", "StrengGeheim", 10001, url);

			// Cases to test:
			int op = 999;
			switch (op) {
				case 101:// GetMETS
					System.out.println("Calling getMETS ...");
					response = ws.getMETS(urn);
					break;
				case 102:// GetDIP
					System.out.println("Calling getDIP ...");
					response = ws.getDIP(intId);
					break;
				case 103:// GetDIPExternal
					System.out.println("Calling getDIPExternal ...");
					response = ws.getDIPExternal(urn);
					break;
				case 104:// GetFile
					System.out.println("Calling getFile ...");
					response = ws.getFile(intId, file);
					break;
				case 105:// GetModifiedSince
					System.out.println("Calling getModifiedSince ...");
					response = ws.getModifiedSince(date);
					break;
				case 106:// GetHistory
					System.out.println("Calling getHistory ...");
					response = ws.getHistory(urn);
					break;
				case 107:// GetRetrievalStatistics
					System.out.println("Calling getRetrievalStatistics ...");
					response = ws.getRetrievalStatistics();
					break;
				case 108:// ResetRetrievalStatistics
					System.out.println("Calling resetRetrievalStatistics ...");
					response = ws.resetRetrievalStatistics();
					break;
				case 109:// BlockRetrieval
					System.out.println("Calling blockNewRetrievalRequests ...");
					response = ws.blockNewRetrievalRequests();
					break;
				case 110:// AllowRetrieval
					System.out.println("Calling allowRetrieval ...");
					response = ws.allowRetrievalRequests();
					break;
				case 111:// InitCache
					System.out.println("Calling initCache ...");
					response = ws.initCache(rootDir);
					break;
				case 112:// EraseOldCacheEntries
					System.out.println("eraseOldCacheEntries ...");
					response = ws.eraseOldCacheEntries(timeLimit);
					break;
				case 113:// DeleteCache
					System.out.println("Calling deleteCache ...");
					response = ws.deleteCache();
					break;

				case 201:// GetFileFormatsList
					System.out.println("Calling getFileFormatList ...");
					response = ws.getFileFormatList();
					break;
				case 202:// ReloadFileFormatsList
					System.out.println("Calling reloadFileFormatList ...");
					response = ws.reloadFileFormatList();
					break;
				case 203://
					System.out.println("Calling ingestDocument ...");
					CustomXMLType techMd = CustomXMLType.Factory.newInstance();
					// CustomXMLType descMd1 =
					// CustomXMLType.Factory.newInstance();

					/*
					 * XmlOptions options = new XmlOptions();
					 * options.setCompileNoValidation();
					 * options.setCharacterEncoding("UTF-8");
					 * 
					 * //org.w3c.dom.Node root = techMd.newDomNode(options);
					 * techMd.validate(options); System.out.println(lmer +
					 * "\n\n"); techMd.setXmlType("LMER");
					 */
					XmlObject xmlObj = XmlObject.Factory.parse(lmer);
					System.out.println(xmlObj.xmlText());
					techMd.set(xmlObj);
					metadata[0] = techMd;
					response = ws.ingestDocument(policy, metadata);
					break;
				case 204:// ingestSIP
					System.out.println("Calling ingestSIP ...");
					response = ws.ingestSIP(urn, linkToSIP);
					break;
				case 205:// getIngestStatistics
					System.out.println("Calling getIngestStatistics ...");
					response = ws.getIngestStatistics();
					break;
				case 206:// resetIngestStatistics
					System.out.println("Calling resetIngestStatistics ...");
					response = ws.resetIngestStatistics();
					break;
				case 207:// blockNewIngestRequests
					System.out.println("Calling blockNewIngestRequests ...");
					response = ws.blockNewIngestRequests();
					break;
				case 208:// allowIngestRequests
					System.out.println("Calling allowIngestRequests ...");
					response = ws.allowIngestRequests();
					break;

				case 301:// setParameters
					System.out.println("Calling setParameters ...");
					response = ws.setParameters(diasRetrieverURL,
							diasLoaderURL, fileFormatListURL, rootDir,
							protocolRetrievalWS, urlRetrievalWS, tempDir);
					break;
				case 302:// changePassword
					System.out.println("Calling changePassword ...");
					response = ws.changePassword("NeueSehrGeheimeKennwort");
					break;

				case 999:// batch
					// response = ws.blockNewRetrievalRequests();
					System.out.println("Calling initCache ...");
					response = ws.initCache(rootDir);
					System.out.println("Calling allowRetrieval ...");
					response = ws.allowRetrievalRequests();
					/*
					 * System.out.println("Calling getMETS ..."); response =
					 * ws.getMETS(urn); System.out.println("Calling getFile
					 * ..."); response = ws.getFile(intId, file);
					 */
					System.out.println("Calling getDIP ...");
					response = ws.getDIP(intId);
			}

			System.out.println("\nRESPONSE:\n=========");
			System.out.println("Status code:\t" + response.getResponseElem().getStatusCode() + "\n"
					+ "Response text:\t" + response.getResponseElem().getResponseText() + "\n"
					+ "DIAS ID:\t" + response.getResponseElem().getDiasId() + "\n"
					+ "Link to file:\t" + response.getResponseElem().getLinkToFile() + "\n"
					+ "Extra Info:\t" + response.getResponseElem().getExtraInfo() + "\n"
					+ "Error Message:\t" + response.getResponseElem().getErrorMessage() + "\n"
					+ "Wait time:\t" + response.getResponseElem().getWaitTime() + "\n");
		}
		catch (AxisFault af) {
			System.out.println("AXIS FAULT!\n" + af.getMessage() + "\t"
					+ af.getReason());
		}
		catch (Exception e) {
			System.out.println("Exception!\n" + e.getMessage() + "\t"
					+ e.getCause());
			e.printStackTrace();
		}
	}

}
