/*******************************************************************************
 * de.langzeitarchivierung.kopal.ws / WebServiceClient.java
 * 
 * Copyright 2005-2007 by Project kopal
 * 
 * http://kopal.langzeitarchivierung.de/
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307, USA
 * 
 * 
 *******************************************************************************
 *
 *
 * CHANGELOG:
 * 12.07.2007	koLibRI version 1.0
 ******************************************************************************/

package de.langzeitarchivierung.kopal.ws;

// import de.langzeitarchivierung.kopal.retrieval.DiasAccess;
// import de.langzeitarchivierung.kopal.util.WSUtils;
import de.langzeitarchivierung.kopal.ws.wsdl2java.*;

/*******************************************************************************
 * Class to use koLibRI Web Services as a client.
 * 
 * @author Kadir Karaca Koçer, German National Library
 * @version 20070710
 * @see WebServiceServer
 * @since 20070621
 ******************************************************************************/

public class WebServiceClient
{

	// CLASS VARIABLES (Static variables) **********************************

	private KolibriWebServiceStub	retrieval	= null;
	private UserType				user		= UserType.Factory
														.newInstance();

	// private ResponseElemDocument response =
	// ResponseElemDocument.Factory.newInstance();

	// CREATION (Constructors, factory methods, static/inst init) **********

	/***************************************************************************
	 * Constructor
	 * 
	 * @param userName Login name of the user.
	 * @param password User password.
	 * @param institution Number code of the institution this user works for.
	 * @param url URL of Web Service to communicate.
	 * @throws Exception Throws exception if the communication with the server
	 *             does not function properly.
	 * @see KolibriWebServiceStub
	 * @see WebServiceServer
	 * @author Kadir Karaca Koçer, German National Library
	 * @version 20070710
	 * @since 20070630
	 **************************************************************************/
	public WebServiceClient(String userName, String password, int institution,
			String url) throws Exception {
		this.retrieval = new KolibriWebServiceStub(url);
		this.user.setUserName(userName);
		this.user.setPassword(password);
		this.user.setInstitution(institution);
	}

	// GET & SET METHODS **************************************************

	// QUERIES (Queries - no change to object's state) ********************

	// MANIPULATION (Manipulation - what the object does) ******************

	/***************************************************************************
	 * Method to request the packed METS file for a given URN.
	 * 
	 * @param urn URN of the requested object
	 * @return ResponseElemDocument containing all the information needed.
	 * @throws Exception Throws exception if the communication with the server
	 *             does not function properly.
	 * @see GetMETSElemDocument
	 * @see WebServiceServer
	 * @see ResponseElemDocument
	 * @author Kadir Karaca Koçer, German National Library
	 * @version 20070710
	 * @since 20070630
	 **************************************************************************/
	public ResponseElemDocument getMETS(String urn) throws Exception {
		GetMETSElemDocument doc = GetMETSElemDocument.Factory.newInstance();
		// System.out.println("Doc generated!");
		GetMETSElemDocument.GetMETSElem elem = GetMETSElemDocument.GetMETSElem.Factory
				.newInstance();
		elem.setUser(this.user);
		elem.setObjectId(urn);
		doc.setGetMETSElem(elem);

		return this.retrieval.getMETS(doc);
	}

	/***************************************************************************
	 * Method to request a GNU Tarred DIP for a given DIAS Internal number. Note
	 * that this number is different than URN. Because of migrations an URN can
	 * reference more than one DIP and therefore is in DIAS not unique. Only
	 * DIAS internal ID is unique.
	 * 
	 * @param internalId DIAS internal id of the requested object
	 * @return ResponseElemDocument containing all the information needed.
	 * @throws Exception Throws exception if the communication with the server
	 *             does not function properly.
	 * @see GetDIPElemDocument
	 * @see WebServiceServer
	 * @see ResponseElemDocument
	 * @author Kadir Karaca Koçer, German National Library
	 * @version 20070710
	 * @since 20070630
	 **************************************************************************/
	public ResponseElemDocument getDIP(String internalId) throws Exception {
		GetDIPElemDocument doc = GetDIPElemDocument.Factory.newInstance();
		GetDIPElemDocument.GetDIPElem elem = GetDIPElemDocument.GetDIPElem.Factory
				.newInstance();
		elem.setUser(this.user);
		elem.setObjectId(internalId);
		doc.setGetDIPElem(elem);

		return this.retrieval.getDIP(doc);
	}

	/***************************************************************************
	 * Method to request a GNU Tarred DIP for a given DIAS Internal number. Note
	 * that this number is different than URN. Because of migrations an URN can
	 * reference more than one DIP and therefore is in DIAS not unique. Only
	 * DIAS internal ID is unique.
	 * 
	 * This method is for partner institutions. Not for standard retrieval. Will
	 * be implemented in server side as the requests of those institutions.
	 * 
	 * @param internalId DIAS internal id of the requested object
	 * @return ResponseElemDocument containing all the information needed.
	 * @throws Exception Throws exception if the communication with the server
	 *             does not function properly.
	 * @see GetDIPExternalElemDocument
	 * @see WebServiceServer
	 * @see ResponseElemDocument
	 * @author Kadir Karaca Koçer, German National Library
	 * @version 20070710
	 * @since 20070630
	 **************************************************************************/
	public ResponseElemDocument getDIPExternal(String internalId)
			throws Exception {
		GetDIPExternalElemDocument doc = GetDIPExternalElemDocument.Factory.newInstance();
		GetDIPExternalElemDocument.GetDIPExternalElem elem = GetDIPExternalElemDocument.GetDIPExternalElem.Factory.newInstance();
		elem.setUser(this.user);
		elem.setObjectId(internalId);
		doc.setGetDIPExternalElem(elem);

		return this.retrieval.getDIPExternal(doc);
	}

	/***************************************************************************
	 * Method to request a file for a given DIAS Internal number of its AIP.
	 * Note that this number is different than URN. Because of migrations an URN
	 * can reference more than one DIP and therefore is in DIAS not unique. Only
	 * DIAS internal ID is unique.
	 * 
	 * @param internalId DIAS internal id of the requested object
	 * @param fileName The name of the file in this package.
	 * @return ResponseElemDocument containing all the information needed.
	 * @throws Exception Throws exception if the communication with the server
	 *             does not function properly.
	 * @see GetFileElemDocument
	 * @see WebServiceServer
	 * @see ResponseElemDocument
	 * @author Kadir Karaca Koçer, German National Library
	 * @version 20070710
	 * @since 20070630
	 **************************************************************************/
	public ResponseElemDocument getFile(String internalId, String fileName)
			throws Exception {
		GetFileElemDocument doc = GetFileElemDocument.Factory.newInstance();
		GetFileElemDocument.GetFileElem elem = GetFileElemDocument.GetFileElem.Factory.newInstance();
		elem.setUser(this.user);
		elem.setObjectId(internalId);
		elem.setFileName(fileName);
		doc.setGetFileElem(elem);

		return this.retrieval.getFile(doc);
	}

	/***************************************************************************
	 * Method to request list of files which are modified since date X.
	 * 
	 * @param cal The time stamp.
	 * @return ResponseElemDocument containing all the information needed.
	 * @throws Exception Throws exception if the communication with the server
	 *             does not function properly.
	 * @see GetModifiedSinceElemDocument
	 * @see WebServiceServer
	 * @see ResponseElemDocument
	 * @author Kadir Karaca Koçer, German National Library
	 * @version 20070710
	 * @since 20070630
	 **************************************************************************/
	public ResponseElemDocument getModifiedSince(java.util.GregorianCalendar cal)
			throws Exception {
		GetModifiedSinceElemDocument doc = GetModifiedSinceElemDocument.Factory.newInstance();
		GetModifiedSinceElemDocument.GetModifiedSinceElem elem = GetModifiedSinceElemDocument.GetModifiedSinceElem.Factory.newInstance();
		elem.setUser(this.user);
		elem.setModificationdate(cal);
		doc.setGetModifiedSinceElem(elem);

		return this.retrieval.getModifiedSince(doc);
	}

	/***************************************************************************
	 * Method to get the migration history of an object.
	 * 
	 * @param urn URN of the object
	 * @return ResponseElemDocument containing all the information needed.
	 * @throws Exception Throws exception if the communication with the server
	 *             does not function properly.
	 * @see GetMETSElemDocument
	 * @see WebServiceServer
	 * @see ResponseElemDocument
	 * @author Kadir Karaca Koçer, German National Library
	 * @version 20070710
	 * @since 20070630
	 **************************************************************************/
	public ResponseElemDocument getHistory(String urn) throws Exception {
		GetHistoryElemDocument.GetHistoryElem elem
		    = GetHistoryElemDocument.GetHistoryElem.Factory.newInstance();
		elem.setUser(this.user);
		elem.setObjectId(urn);
		GetHistoryElemDocument doc = GetHistoryElemDocument.Factory.newInstance();
		doc.setGetHistoryElem(elem);

		return this.retrieval.getHistory(doc);
	}

	/***************************************************************************
	 * Blocks new retrieval requests. Good for maintenance or before shutting
	 * down the server.
	 * 
	 * @return ResponseElemDocument containing all the information needed.
	 * @throws Exception Throws exception if the communication with the server
	 *             does not function properly.
	 * @see BlockNewRetrievalRequestsElemDocument
	 * @see WebServiceServer
	 * @see ResponseElemDocument
	 * @author Kadir Karaca Koçer, German National Library
	 * @version 20070710
	 * @since 20070630
	 **************************************************************************/
	public ResponseElemDocument blockNewRetrievalRequests() throws Exception {
		BlockNewRetrievalRequestsElemDocument.BlockNewRetrievalRequestsElem elem =
			BlockNewRetrievalRequestsElemDocument.BlockNewRetrievalRequestsElem.Factory.newInstance();
		elem.setUser(this.user);

		BlockNewRetrievalRequestsElemDocument doc =
			BlockNewRetrievalRequestsElemDocument.Factory.newInstance();
		doc.setBlockNewRetrievalRequestsElem(elem);

		return this.retrieval.blockNewRetrievalRequests(doc);
	}

	/***************************************************************************
	 * Allow new retrieval requests. Good after maintenance or before using the
	 * server first time.
	 * 
	 * @return ResponseElemDocument containing all the information needed.
	 * @throws Exception Throws exception if the communication with the server
	 *             does not function properly.
	 * @see AllowRetrievalRequestsElemDocument
	 * @see WebServiceServer
	 * @see ResponseElemDocument
	 * @author Kadir Karaca Koçer, German National Library
	 * @version 20070710
	 * @since 20070630
	 **************************************************************************/
	public ResponseElemDocument allowRetrievalRequests() throws Exception {
		AllowRetrievalRequestsElemDocument.AllowRetrievalRequestsElem elem =
			AllowRetrievalRequestsElemDocument.AllowRetrievalRequestsElem.Factory.newInstance();
		elem.setUser(this.user);

		AllowRetrievalRequestsElemDocument doc =
			AllowRetrievalRequestsElemDocument.Factory.newInstance();
		doc.setAllowRetrievalRequestsElem(elem);

		return this.retrieval.allowRetrievalRequests(doc);
	}

	/***************************************************************************
	 * Initializes the cache.
	 * 
	 * @param cacheRootDir Root directory of cache.
	 * @return ResponseElemDocument containing all the information needed.
	 * @throws Exception Throws exception if the communication with the server
	 *             does not function properly.
	 * @see InitCacheElemDocument
	 * @see WebServiceServer
	 * @see ResponseElemDocument
	 * @author Kadir Karaca Koçer, German National Library
	 * @version 20070710
	 * @since 20070630
	 **************************************************************************/
	public ResponseElemDocument initCache(String cacheRootDir) throws Exception {
		InitCacheElemDocument.InitCacheElem elem =
			InitCacheElemDocument.InitCacheElem.Factory.newInstance();
		elem.setUser(this.user);
		elem.setRootdir(cacheRootDir);
		InitCacheElemDocument doc = InitCacheElemDocument.Factory.newInstance();
		doc.setInitCacheElem(elem);

		return this.retrieval.initCache(doc);
	}

	/***************************************************************************
	 * Deletes the old files in the cache.
	 * 
	 * @param timeLimit Number of days how old the files at least must be to
	 *            delete.
	 * @return ResponseElemDocument containing all the information needed.
	 * @throws Exception Throws exception if the communication with the server
	 *             does not function properly.
	 * @see EraseOldCacheEntriesElemDocument
	 * @see WebServiceServer
	 * @see ResponseElemDocument
	 * @author Kadir Karaca Koçer, German National Library
	 * @version 20070710
	 * @since 20070630
	 **************************************************************************/
	public ResponseElemDocument eraseOldCacheEntries(int timeLimit)
			throws Exception {
		EraseOldCacheEntriesElemDocument.EraseOldCacheEntriesElem elem =
			EraseOldCacheEntriesElemDocument.EraseOldCacheEntriesElem.Factory.newInstance();
		elem.setUser(this.user);
		elem.setTimelimit(timeLimit);
		EraseOldCacheEntriesElemDocument doc =
			EraseOldCacheEntriesElemDocument.Factory.newInstance();
		doc.setEraseOldCacheEntriesElem(elem);

		return this.retrieval.eraseOldCacheEntries(doc);
	}

	/***************************************************************************
	 * Deletes the whole cache.
	 * 
	 * @return ResponseElemDocument containing all the information needed.
	 * @throws Exception Throws exception if the communication with the server
	 *             does not function properly.
	 * @see DeleteCacheElemDocument
	 * @see WebServiceServer
	 * @see ResponseElemDocument
	 * @author Kadir Karaca Koçer, German National Library
	 * @version 20070710
	 * @since 20070630
	 **************************************************************************/
	public ResponseElemDocument deleteCache() throws Exception {
		DeleteCacheElemDocument.DeleteCacheElem elem =
			DeleteCacheElemDocument.DeleteCacheElem.Factory.newInstance();
		elem.setUser(this.user);
		DeleteCacheElemDocument doc =
			DeleteCacheElemDocument.Factory.newInstance();
		doc.setDeleteCacheElem(elem);

		return this.retrieval.deleteCache(doc);
	}

	/***************************************************************************
	 * Delivers statistical data about Retrieval.
	 * 
	 * @return ResponseElemDocument containing all the information needed.
	 * @throws Exception Throws exception if the communication with the server
	 *             does not function properly.
	 * @see GetRetrievalStatisticsElemDocument
	 * @see WebServiceServer
	 * @see ResponseElemDocument
	 * @author Kadir Karaca Koçer, German National Library
	 * @version 20070710
	 * @since 20070630
	 **************************************************************************/
	public ResponseElemDocument getRetrievalStatistics() throws Exception {
		GetRetrievalStatisticsElemDocument.GetRetrievalStatisticsElem elem =
			GetRetrievalStatisticsElemDocument.GetRetrievalStatisticsElem.Factory.newInstance();
		elem.setUser(this.user);
		GetRetrievalStatisticsElemDocument doc =
			GetRetrievalStatisticsElemDocument.Factory.newInstance();
		doc.setGetRetrievalStatisticsElem(elem);

		return this.retrieval.getRetrievalStatistics(doc);
	}

	/***************************************************************************
	 * Resets the statistical data.
	 * 
	 * @return ResponseElemDocument containing all the information needed.
	 * @throws Exception Throws exception if the communication with the server
	 *             does not function properly.
	 * @see ResetRetrievalStatisticsElemDocument
	 * @see WebServiceServer
	 * @see ResponseElemDocument
	 * @author Kadir Karaca Koçer, German National Library
	 * @version 20070710
	 * @since 20070630
	 **************************************************************************/
	public ResponseElemDocument resetRetrievalStatistics() throws Exception {
		ResetRetrievalStatisticsElemDocument.ResetRetrievalStatisticsElem elem =
			ResetRetrievalStatisticsElemDocument.ResetRetrievalStatisticsElem.Factory.newInstance();
		elem.setUser(this.user);
		ResetRetrievalStatisticsElemDocument doc =
			ResetRetrievalStatisticsElemDocument.Factory.newInstance();
		doc.setResetRetrievalStatisticsElem(elem);

		return this.retrieval.resetRetrievalStatistics(doc);
	}


	// I N G E S T

	/***************************************************************************
	 * Gets a copy of the official list of file formats that defined in DIAS.
	 * 
	 * @return ResponseElemDocument containing all the information needed.
	 * @throws Exception Throws exception if the communication with the server
	 *             does not function properly.
	 * @see GetFileFormatListElemDocument
	 * @see WebServiceServer
	 * @see ResponseElemDocument
	 * @author Kadir Karaca Koçer, German National Library
	 * @version 20070710
	 * @since 20070630
	 **************************************************************************/
	public ResponseElemDocument getFileFormatList() throws Exception {
		GetFileFormatListElemDocument.GetFileFormatListElem elem =
			GetFileFormatListElemDocument.GetFileFormatListElem.Factory.newInstance();
		elem.setUser(this.user);
		GetFileFormatListElemDocument doc =
			GetFileFormatListElemDocument.Factory.newInstance();
		doc.setGetFileFormatListElem(elem);

		return this.retrieval.getFileFormatList(doc);
	}

	/***************************************************************************
	 * Reloads the official list of file formats that defined in DIAS and saves
	 * on server.
	 * 
	 * @return ResponseElemDocument containing all the information needed.
	 * @throws Exception Throws exception if the communication with the server
	 *             does not function properly.
	 * @see ReloadFileFormatListElemDocument
	 * @see WebServiceServer
	 * @see ResponseElemDocument
	 * @author Kadir Karaca Koçer, German National Library
	 * @version 20070710
	 * @since 20070630
	 **************************************************************************/
	public ResponseElemDocument reloadFileFormatList() throws Exception {
		ReloadFileFormatListElemDocument.ReloadFileFormatListElem elem =
			ReloadFileFormatListElemDocument.ReloadFileFormatListElem.Factory.newInstance();
		elem.setUser(this.user);
		ReloadFileFormatListElemDocument doc =
			ReloadFileFormatListElemDocument.Factory.newInstance();
		doc.setReloadFileFormatListElem(elem);

		return this.retrieval.reloadFileFormatList(doc);
	}

	/***************************************************************************
	 * Ingests a digital document into DIAS. (Not fully implemented on the
	 * server side!)
	 * 
	 * @param policy The policy to use.
	 * @param metadata 1 to 6 Custom XMLs defining this digital document. At the
	 *            moment 1 LMER and 1 Dublin Core others will be copied verbatim
	 *            in METS file.
	 * @return ResponseElemDocument containing all the information needed.
	 * @throws Exception Throws exception if the communication with the server
	 *             does not function properly.
	 * @see IngestDocumentElemDocument
	 * @see WebServiceServer
	 * @see ResponseElemDocument
	 * @author Kadir Karaca Koçer, German National Library
	 * @version 20070710
	 * @since 20070630
	 **************************************************************************/
	public ResponseElemDocument ingestDocument(String policy,
			CustomXMLType[] metadata) throws Exception {
		IngestDocumentElemDocument.IngestDocumentElem elem =
			IngestDocumentElemDocument.IngestDocumentElem.Factory.newInstance();
		elem.setUser(this.user);
		elem.setPolicy(policy);
		elem.setMetadataArray(metadata);
		IngestDocumentElemDocument doc = IngestDocumentElemDocument.Factory
				.newInstance();
		doc.setIngestDocumentElem(elem);

		return this.retrieval.ingestDocument(doc);
	}

	/***************************************************************************
	 * Ingests a ready made SIP into DIAS. Usefull for external institutions
	 * (Will be implemented in the next version!)
	 * 
	 * @param urn Persistent identifier of this SIP.
	 * @param linkToSIP URL of the SIP file.
	 * @return ResponseElemDocument containing all the information needed.
	 * @throws Exception Throws exception if the communication with the server
	 *             does not function properly.
	 * @see IngestSIPElemDocument
	 * @see WebServiceServer
	 * @see ResponseElemDocument
	 * @author Kadir Karaca Koçer, German National Library
	 * @version 20070710
	 * @since 20070630
	 **************************************************************************/
	public ResponseElemDocument ingestSIP(String urn, String linkToSIP)
			throws Exception {
		IngestSIPElemDocument.IngestSIPElem elem = IngestSIPElemDocument.IngestSIPElem.Factory
				.newInstance();
		elem.setUser(this.user);
		elem.setUrn(urn);
		elem.setLinkToSIP(linkToSIP);
		IngestSIPElemDocument doc = IngestSIPElemDocument.Factory.newInstance();
		doc.setIngestSIPElem(elem);

		return this.retrieval.ingestSIP(doc);
	}

	/***************************************************************************
	 * Blocks the new ingest requests. Good for administration.
	 * 
	 * @return ResponseElemDocument containing all the information needed.
	 * @throws Exception Throws exception if the communication with the server
	 *             does not function properly.
	 * @see BlockNewIngestRequestsElemDocument
	 * @see WebServiceServer
	 * @see ResponseElemDocument
	 * @author Kadir Karaca Koçer, German National Library
	 * @version 20070710
	 * @since 20070630
	 **************************************************************************/
	public ResponseElemDocument blockNewIngestRequests() throws Exception {
		BlockNewIngestRequestsElemDocument.BlockNewIngestRequestsElem elem = BlockNewIngestRequestsElemDocument.BlockNewIngestRequestsElem.Factory
				.newInstance();
		elem.setUser(this.user);
		BlockNewIngestRequestsElemDocument doc = BlockNewIngestRequestsElemDocument.Factory
				.newInstance();
		doc.setBlockNewIngestRequestsElem(elem);

		return this.retrieval.blockNewIngestRequests(doc);
	}

	/***************************************************************************
	 * Allows the new ingest requests. Must be called at start and after
	 * administration.
	 * 
	 * @return ResponseElemDocument containing all the information needed.
	 * @throws Exception Throws exception if the communication with the server
	 *             does not function properly.
	 * @see AllowIngestRequestsElemDocument
	 * @see WebServiceServer
	 * @see ResponseElemDocument
	 * @author Kadir Karaca Koçer, German National Library
	 * @version 20070710
	 * @since 20070630
	 **************************************************************************/
	public ResponseElemDocument allowIngestRequests() throws Exception {
		AllowIngestRequestsElemDocument.AllowIngestRequestsElem elem = AllowIngestRequestsElemDocument.AllowIngestRequestsElem.Factory
				.newInstance();
		elem.setUser(this.user);
		AllowIngestRequestsElemDocument doc = AllowIngestRequestsElemDocument.Factory
				.newInstance();
		doc.setAllowIngestRequestsElem(elem);

		return this.retrieval.allowIngestRequests(doc);
	}

	/***************************************************************************
	 * Gets some statistics about the total ingest activities. (Will be
	 * implemented in the next version!)
	 * 
	 * @return ResponseElemDocument containing all the information needed.
	 * @throws Exception Throws exception if the communication with the server
	 *             does not function properly.
	 * @see GetIngestStatisticsElemDocument
	 * @see WebServiceServer
	 * @see ResponseElemDocument
	 * @author Kadir Karaca Koçer, German National Library
	 * @version 20070710
	 * @since 20070630
	 **************************************************************************/
	public ResponseElemDocument getIngestStatistics() throws Exception {
		GetIngestStatisticsElemDocument.GetIngestStatisticsElem elem = GetIngestStatisticsElemDocument.GetIngestStatisticsElem.Factory
				.newInstance();
		elem.setUser(this.user);
		GetIngestStatisticsElemDocument doc = GetIngestStatisticsElemDocument.Factory
				.newInstance();
		doc.setGetIngestStatisticsElem(elem);

		return this.retrieval.getIngestStatistics(doc);
	}

	/***************************************************************************
	 * Resets the ingest statistics. (Will be implemented in the next version!)
	 * 
	 * @return ResponseElemDocument containing all the information needed.
	 * @throws Exception Throws exception if the communication with the server
	 *             does not function properly.
	 * @see ResetIngestStatisticsElemDocument
	 * @see WebServiceServer
	 * @see ResponseElemDocument
	 * @author Kadir Karaca Koçer, German National Library
	 * @version 20070710
	 * @since 20070630
	 **************************************************************************/
	public ResponseElemDocument resetIngestStatistics() throws Exception {
		ResetIngestStatisticsElemDocument.ResetIngestStatisticsElem elem = ResetIngestStatisticsElemDocument.ResetIngestStatisticsElem.Factory
				.newInstance();
		elem.setUser(this.user);
		ResetIngestStatisticsElemDocument doc = ResetIngestStatisticsElemDocument.Factory
				.newInstance();
		doc.setResetIngestStatisticsElem(elem);

		return this.retrieval.resetIngestStatistics(doc);
	}

	/***************************************************************************
	 * Sets the working parameters of web service.
	 * 
	 * @param dr DIAS Retriever URL.
	 * @param dl DIAS Loader URL.
	 * @param ffl File format list URL.
	 * @param crd Cache root directory.
	 * @param rp Protocol used by retrieval web service.
	 * @param rURL URL of retrieval web service
	 * @param tempDir Directory for temporar files.
	 * @return ResponseElemDocument containing all the information needed.
	 * @throws Exception Throws exception if the communication with the server
	 *             does not function properly.
	 * @see SetParametersElemDocument
	 * @see WebServiceServer
	 * @see ResponseElemDocument
	 * @author Kadir Karaca Koçer, German National Library
	 * @version 20070710
	 * @since 20070630
	 **************************************************************************/
	public ResponseElemDocument setParameters(String dr, String dl, String ffl,
			String crd, String rp, String rURL, String tempDir)
			throws Exception {
		SetParametersElemDocument.SetParametersElem elem = SetParametersElemDocument.SetParametersElem.Factory
				.newInstance();
		// Set the parameters
		elem.setUser(this.user);
		if ((dr != null) && (!dr.equals("")))
			elem.setDiasRetrieverURL(dr);
		if ((dl != null) && (!dl.equals("")))
			elem.setDiasLoaderURL(dl);
		if ((ffl != null) && (!ffl.equals("")))
			elem.setFileFormatListURL(ffl);
		if ((crd != null) && (!crd.equals("")))
			elem.setCacheRootDir(crd);
		if ((rp != null) && (!rp.equals("")))
			elem.setProtocolRetrievalWS(rp);
		if ((rURL != null) && (!rURL.equals("")))
			elem.setUrlRetrievalWS(rURL);
		if ((tempDir != null) && (!tempDir.equals("")))
			elem.setTempDir(tempDir);

		SetParametersElemDocument doc = SetParametersElemDocument.Factory
				.newInstance();
		doc.setSetParametersElem(elem);

		return this.retrieval.setParameters(doc);
	}

	/***************************************************************************
	 * Change the user password. (User management will be implemented in a later
	 * version.)
	 * 
	 * @param newPassword New password.
	 * @return ResponseElemDocument containing all the information needed.
	 * @throws Exception Throws exception if the communication with the server
	 *             does not function properly.
	 * @see GetIngestStatisticsElemDocument
	 * @see WebServiceServer
	 * @see ResponseElemDocument
	 * @author Kadir Karaca Koçer, German National Library
	 * @version 20070710
	 * @since 20070630
	 **************************************************************************/
	public ResponseElemDocument changePassword(String newPassword)
			throws Exception {
		ChangePasswordElemDocument.ChangePasswordElem elem = ChangePasswordElemDocument.ChangePasswordElem.Factory
				.newInstance();
		elem.setUser(this.user);
		elem.setNewPassword(newPassword);
		ChangePasswordElemDocument doc = ChangePasswordElemDocument.Factory
				.newInstance();
		doc.setChangePasswordElem(elem);

		return this.retrieval.changePassword(doc);
	}

}
